import sys
from _typeshed import ReadableBuffer, Unused
from types import TracebackType
from typing import Any, Final, Literal, final, overload
from typing_extensions import Self, TypeAlias

if sys.platform == "win32":
    _KeyType: TypeAlias = HKEYType | int
    def CloseKey(hkey: _KeyType, /) -> None: ...
    def ConnectRegistry(computer_name: str | None, key: _KeyType, /) -> HKEYType: ...
    def CreateKey(key: _KeyType, sub_key: str | None, /) -> HKEYType: ...
    def CreateKeyEx(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131078) -> HKEYType: ...
    def DeleteKey(key: _KeyType, sub_key: str, /) -> None: ...
    def DeleteKeyEx(key: _KeyType, sub_key: str, access: int = 256, reserved: int = 0) -> None: ...
    def DeleteValue(key: _KeyType, value: str, /) -> None: ...
    def EnumKey(key: _KeyType, index: int, /) -> str: ...
    def EnumValue(key: _KeyType, index: int, /) -> tuple[str, Any, int]: ...
    def ExpandEnvironmentStrings(string: str, /) -> str: ...
    def FlushKey(key: _KeyType, /) -> None: ...
    def LoadKey(key: _KeyType, sub_key: str, file_name: str, /) -> None: ...
    def OpenKey(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131097) -> HKEYType: ...
    def OpenKeyEx(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131097) -> HKEYType: ...
    def QueryInfoKey(key: _KeyType, /) -> tuple[int, int, int]: ...
    def QueryValue(key: _KeyType, sub_key: str | None, /) -> str: ...
    def QueryValueEx(key: _KeyType, name: str, /) -> tuple[Any, int]: ...
    def SaveKey(key: _KeyType, file_name: str, /) -> None: ...
    def SetValue(key: _KeyType, sub_key: str | None, type: int, value: str, /) -> None: ...
    @overload  # type=REG_DWORD|REG_QWORD
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[4, 5], value: int | None, /
    ) -> None: ...
    @overload  # type=REG_SZ|REG_EXPAND_SZ
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[1, 2], value: str | None, /
    ) -> None: ...
    @overload  # type=REG_MULTI_SZ
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[7], value: list[str] | None, /
    ) -> None: ...
    @overload  # type=REG_BINARY and everything else
    def SetValueEx(
        key: _KeyType,
        value_name: str | None,
        reserved: Unused,
        type: Literal[0, 3, 8, 9, 10, 11],
        value: ReadableBuffer | None,
        /,
    ) -> None: ...
    @overload  # Unknown or undocumented
    def SetValueEx(
        key: _KeyType,
        value_name: str | None,
        reserved: Unused,
        type: int,
        value: int | str | list[str] | ReadableBuffer | None,
        /,
    ) -> None: ...
    def DisableReflectionKey(key: _KeyType, /) -> None: ...
    def EnableReflectionKey(key: _KeyType, /) -> None: ...
    def QueryReflectionKey(key: _KeyType, /) -> bool: ...

    HKEY_CLASSES_ROOT: Final[int]
    HKEY_CURRENT_USER: Final[int]
    HKEY_LOCAL_MACHINE: Final[int]
    HKEY_USERS: Final[int]
    HKEY_PERFORMANCE_DATA: Final[int]
    HKEY_CURRENT_CONFIG: Final[int]
    HKEY_DYN_DATA: Final[int]

    KEY_ALL_ACCESS: Final = 983103
    KEY_WRITE: Final = 131078
    KEY_READ: Final = 131097
    KEY_EXECUTE: Final = 131097
    KEY_QUERY_VALUE: Final = 1
    KEY_SET_VALUE: Final = 2
    KEY_CREATE_SUB_KEY: Final = 4
    KEY_ENUMERATE_SUB_KEYS: Final = 8
    KEY_NOTIFY: Final = 16
    KEY_CREATE_LINK: Final = 32

    KEY_WOW64_64KEY: Final = 256
    KEY_WOW64_32KEY: Final = 512

    REG_BINARY: Final = 3
    REG_DWORD: Final = 4
    REG_DWORD_LITTLE_ENDIAN: Final = 4
    REG_DWORD_BIG_ENDIAN: Final = 5
    REG_EXPAND_SZ: Final = 2
    REG_LINK: Final = 6
    REG_MULTI_SZ: Final = 7
    REG_NONE: Final = 0
    REG_QWORD: Final = 11
    REG_QWORD_LITTLE_ENDIAN: Final = 11
    REG_RESOURCE_LIST: Final = 8
    REG_FULL_RESOURCE_DESCRIPTOR: Final = 9
    REG_RESOURCE_REQUIREMENTS_LIST: Final = 10
    REG_SZ: Final = 1

    REG_CREATED_NEW_KEY: Final = 1  # undocumented
    REG_LEGAL_CHANGE_FILTER: Final = 268435471  # undocumented
    REG_LEGAL_OPTION: Final = 31  # undocumented
    REG_NOTIFY_CHANGE_ATTRIBUTES: Final = 2  # undocumented
    REG_NOTIFY_CHANGE_LAST_SET: Final = 4  # undocumented
    REG_NOTIFY_CHANGE_NAME: Final = 1  # undocumented
    REG_NOTIFY_CHANGE_SECURITY: Final = 8  # undocumented
    REG_NO_LAZY_FLUSH: Final = 4  # undocumented
    REG_OPENED_EXISTING_KEY: Final = 2  # undocumented
    REG_OPTION_BACKUP_RESTORE: Final = 4  # undocumented
    REG_OPTION_CREATE_LINK: Final = 2  # undocumented
    REG_OPTION_NON_VOLATILE: Final = 0  # undocumented
    REG_OPTION_OPEN_LINK: Final = 8  # undocumented
    REG_OPTION_RESERVED: Final = 0  # undocumented
    REG_OPTION_VOLATILE: Final = 1  # undocumented
    REG_REFRESH_HIVE: Final = 2  # undocumented
    REG_WHOLE_HIVE_VOLATILE: Final = 1  # undocumented

    error = OSError

    # Though this class has a __name__ of PyHKEY, it's exposed as HKEYType for some reason
    @final
    class HKEYType:
        def __bool__(self) -> bool: ...
        def __int__(self) -> int: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None, /
        ) -> bool | None: ...
        def Close(self) -> None: ...
        def Detach(self) -> int: ...
        def __hash__(self) -> int: ...
        @property
        def handle(self) -> int: ...



from authlib.oauth2.rfc6749 import TokenValidator

class BearerTokenValidator(TokenValidator):
    TOKEN_TYPE: str
    def authenticate_token(self, token_string) -> None: ...
    def validate_token(self, token, scopes, request) -> None: ...



from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class ExceptionMixin:
    def raises(self, ex: type[BaseException] | BaseException) -> Self: ...
    # The types of some_args and some_kwargs must equal the types of the called function.
    def when_called_with(self, *some_args: Any, **some_kwargs: Any) -> Self: ...



from collections.abc import Iterable, Sequence

from ._types import ErrorCorrect
from .image.base import BaseImage, DrawerAliases as DrawerAliases

default_factories: dict[str, str]
error_correction: dict[str, ErrorCorrect]

def main(args: Sequence[str] | None = None) -> None: ...
def get_factory(module: str) -> type[BaseImage]: ...
def get_drawer_help() -> str: ...
def commas(items: Iterable[str], joiner: str = "or") -> str: ...



from argparse import ArgumentParser

from ..options.manager import OptionManager

def stage1_arg_parser() -> ArgumentParser: ...

class JobsArgument:
    is_auto: bool
    n_jobs: int
    def __init__(self, arg: str) -> None: ...

def register_default_options(option_manager: OptionManager) -> None: ...



class ExecApiMixin:
    def exec_create(
        self,
        container,
        cmd,
        stdout: bool = True,
        stderr: bool = True,
        stdin: bool = False,
        tty: bool = False,
        privileged: bool = False,
        user: str = "",
        environment=None,
        workdir=None,
        detach_keys=None,
    ): ...
    def exec_inspect(self, exec_id): ...
    def exec_resize(self, exec_id, height=None, width=None) -> None: ...
    def exec_start(
        self, exec_id, detach: bool = False, tty: bool = False, stream: bool = False, socket: bool = False, demux: bool = False
    ): ...



from collections.abc import Callable
from typing import Any, overload
from typing_extensions import deprecated

from numpy.typing import ArrayLike, NDArray

from .utils import _Seed

@overload
def bootstrap(
    *args: ArrayLike,
    n_boot: int = 10000,
    func: str | Callable[..., Any] = "mean",
    axis: int | None = None,
    units: ArrayLike | None = None,
    seed: _Seed | None = None,
) -> NDArray[Any]: ...
@overload
@deprecated("Parameter `random_seed` is deprecated in favor of `seed`")
def bootstrap(
    *args: ArrayLike,
    n_boot: int = 10000,
    func: str | Callable[..., Any] = "mean",
    axis: int | None = None,
    units: ArrayLike | None = None,
    seed: _Seed | None = None,
    random_seed: _Seed | None = None,
) -> NDArray[Any]: ...



from collections.abc import Callable
from typing import ClassVar, Final
from xml.sax.handler import ContentHandler
from xml.sax.xmlreader import Locator, XMLReader

import docutils
from docutils import nodes, writers
from docutils.frontend import Values

__docformat__: Final = "reStructuredText"

class RawXmlError(docutils.ApplicationError): ...

class Writer(writers.Writer[str]):
    settings_defaults: ClassVar[dict[str, str]]
    config_section: ClassVar[str]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    translator_class: type[XMLTranslator]
    visitor: XMLTranslator

class XMLTranslator(nodes.GenericNodeVisitor):
    doctype: ClassVar[str]
    generator: ClassVar[str]
    xmlparser: ClassVar[XMLReader]
    warn: Callable[..., nodes.system_message]
    error: Callable[..., nodes.system_message]
    settings: Values
    indent: str
    newline: str
    level: int
    in_simple: int
    fixed_text: int
    output: list[str]
    the_handle: TestXml
    def __init__(self, document: nodes.document) -> None: ...
    simple_nodes: ClassVar[tuple[type[nodes.Element], ...]]
    def default_visit(self, node: nodes.Element) -> None: ...  # type: ignore[override]
    def default_departure(self, node: nodes.Element) -> None: ...  # type: ignore[override]
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...

class TestXml(ContentHandler):
    locator: Locator
    def setDocumentLocator(self, locator: Locator) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..cmd import Command

class bdist_rpm(Command):
    description: ClassVar[str]
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    negative_opt: ClassVar[dict[str, str]]
    bdist_base: Incomplete
    rpm_base: Incomplete
    dist_dir: Incomplete
    python: Incomplete
    fix_python: Incomplete
    spec_only: Incomplete
    binary_only: Incomplete
    source_only: Incomplete
    use_bzip2: Incomplete
    distribution_name: Incomplete
    group: Incomplete
    release: Incomplete
    serial: Incomplete
    vendor: Incomplete
    packager: Incomplete
    doc_files: Incomplete
    changelog: Incomplete
    icon: Incomplete
    prep_script: Incomplete
    build_script: Incomplete
    install_script: Incomplete
    clean_script: Incomplete
    verify_script: Incomplete
    pre_install: Incomplete
    post_install: Incomplete
    pre_uninstall: Incomplete
    post_uninstall: Incomplete
    prep: Incomplete
    provides: Incomplete
    requires: Incomplete
    conflicts: Incomplete
    build_requires: Incomplete
    obsoletes: Incomplete
    keep_temp: bool
    use_rpm_opt_flags: bool
    rpm3_mode: bool
    no_autoreq: bool
    force_arch: Incomplete
    quiet: bool
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def finalize_package_data(self) -> None: ...
    def run(self) -> None: ...



"""High level strategies for converters."""

from ._class_methods import use_class_methods
from ._subclasses import include_subclasses
from ._unions import configure_tagged_union, configure_union_passthrough

__all__ = [
    "configure_tagged_union",
    "configure_union_passthrough",
    "include_subclasses",
    "use_class_methods",
]



from logging import Logger

from oauthlib.common import Request

from ..tokens import TokenBase
from .base import GrantTypeBase

log: Logger

class ResourceOwnerPasswordCredentialsGrant(GrantTypeBase):
    def create_token_response(self, request: Request, token_handler: TokenBase) -> tuple[dict[str, str], str, int | None]: ...
    def validate_token_request(self, request: Request) -> None: ...



"""Utilities that are imported by multiple scripts in the tests directory."""

from __future__ import annotations

import functools
import re
import sys
import tempfile
from collections.abc import Iterable, Mapping
from pathlib import Path
from types import MethodType
from typing import TYPE_CHECKING, Any, Final, NamedTuple
from typing_extensions import TypeAlias

import pathspec
from packaging.requirements import Requirement

from .paths import GITIGNORE_PATH, REQUIREMENTS_PATH, STDLIB_PATH, STUBS_PATH, TEST_CASES_DIR, allowlists_path, test_cases_path

if TYPE_CHECKING:
    from _typeshed import OpenTextMode

try:
    from termcolor import colored as colored  # pyright: ignore[reportAssignmentType]
except ImportError:

    def colored(text: str, color: str | None = None, **kwargs: Any) -> str:  # type: ignore[misc] # noqa: ARG001
        return text


PYTHON_VERSION: Final = f"{sys.version_info.major}.{sys.version_info.minor}"


def strip_comments(text: str) -> str:
    return text.split("#")[0].strip()


# ====================================================================
# Printing utilities
# ====================================================================


def print_command(cmd: str | Iterable[str]) -> None:
    if not isinstance(cmd, str):
        cmd = " ".join(cmd)
    print(colored(f"Running: {cmd}", "blue"))


def print_info(message: str) -> None:
    print(colored(message, "blue"))


def print_warning(message: str) -> None:
    print(colored(message, "yellow"))


def print_error(error: str, end: str = "\n", fix_path: tuple[str, str] = ("", "")) -> None:
    error_split = error.split("\n")
    old, new = fix_path
    for line in error_split[:-1]:
        print(colored(line.replace(old, new), "red"))
    print(colored(error_split[-1], "red"), end=end)


def print_success_msg() -> None:
    print(colored("success", "green"))


def print_divider() -> None:
    """Print a row of * symbols across the screen.

    This can be useful to divide terminal output into separate sections.
    """
    print()
    print("*" * 70)
    print()


def print_time(t: float) -> None:
    print(f"({t:.2f} s) ", end="")


# ====================================================================
# Dynamic venv creation
# ====================================================================


@functools.cache
def venv_python(venv_dir: Path) -> Path:
    if sys.platform == "win32":
        return venv_dir / "Scripts" / "python.exe"
    return venv_dir / "bin" / "python"


# ====================================================================
# Parsing the requirements file
# ====================================================================


@functools.cache
def parse_requirements() -> Mapping[str, Requirement]:
    """Return a dictionary of requirements from the requirements file."""
    with REQUIREMENTS_PATH.open(encoding="UTF-8") as requirements_file:
        stripped_lines = map(strip_comments, requirements_file)
        stripped_more = [li for li in stripped_lines if not li.startswith("-")]
        requirements = map(Requirement, filter(None, stripped_more))
        return {requirement.name: requirement for requirement in requirements}


def get_mypy_req() -> str:
    return str(parse_requirements()["mypy"])


# ====================================================================
# Parsing the stdlib/VERSIONS file
# ====================================================================

VersionTuple: TypeAlias = tuple[int, int]
SupportedVersionsDict: TypeAlias = dict[str, tuple[VersionTuple, VersionTuple]]

VERSIONS_PATH = STDLIB_PATH / "VERSIONS"
VERSION_LINE_RE = re.compile(r"^([a-zA-Z_][a-zA-Z0-9_.]*): ([23]\.\d{1,2})-([23]\.\d{1,2})?$")
VERSION_RE = re.compile(r"^([23])\.(\d+)$")


def parse_stdlib_versions_file() -> SupportedVersionsDict:
    result: dict[str, tuple[VersionTuple, VersionTuple]] = {}
    with VERSIONS_PATH.open(encoding="UTF-8") as f:
        for line in f:
            stripped_line = strip_comments(line)
            if stripped_line == "":
                continue
            m = VERSION_LINE_RE.match(stripped_line)
            assert m, f"invalid VERSIONS line: {stripped_line}"
            mod: str = m.group(1)
            assert mod not in result, f"Duplicate module {mod} in VERSIONS"
            min_version = _parse_version(m.group(2))
            max_version = _parse_version(m.group(3)) if m.group(3) else (99, 99)
            result[mod] = min_version, max_version
    return result


def supported_versions_for_module(module_versions: SupportedVersionsDict, module_name: str) -> tuple[VersionTuple, VersionTuple]:
    while "." in module_name:
        if module_name in module_versions:
            return module_versions[module_name]
        module_name = ".".join(module_name.split(".")[:-1])
    return module_versions[module_name]


def _parse_version(v_str: str) -> tuple[int, int]:
    m = VERSION_RE.match(v_str)
    assert m, f"invalid version: {v_str}"
    return int(m.group(1)), int(m.group(2))


# ====================================================================
# Test Directories
# ====================================================================


class DistributionTests(NamedTuple):
    name: str
    test_cases_path: Path

    @property
    def is_stdlib(self) -> bool:
        return self.name == "stdlib"


def distribution_info(distribution_name: str) -> DistributionTests:
    if distribution_name == "stdlib":
        return DistributionTests("stdlib", test_cases_path("stdlib"))
    test_path = test_cases_path(distribution_name)
    if test_path.is_dir():
        if not list(test_path.iterdir()):
            raise RuntimeError(f"{distribution_name!r} has a '{TEST_CASES_DIR}' directory but it is empty!")
        return DistributionTests(distribution_name, test_path)
    raise RuntimeError(f"No test cases found for {distribution_name!r}!")


def get_all_testcase_directories() -> list[DistributionTests]:
    testcase_directories: list[DistributionTests] = []
    for distribution_path in STUBS_PATH.iterdir():
        try:
            pkg_info = distribution_info(distribution_path.name)
        except RuntimeError:
            continue
        testcase_directories.append(pkg_info)
    return [distribution_info("stdlib"), *sorted(testcase_directories)]


def allowlists(distribution_name: str) -> list[str]:
    prefix = "" if distribution_name == "stdlib" else "stubtest_allowlist_"
    version_id = f"py{sys.version_info.major}{sys.version_info.minor}"

    platform_allowlist = f"{prefix}{sys.platform}.txt"
    version_allowlist = f"{prefix}{version_id}.txt"
    combined_allowlist = f"{prefix}{sys.platform}-{version_id}.txt"
    local_version_allowlist = version_allowlist + ".local"

    if distribution_name == "stdlib":
        return ["common.txt", platform_allowlist, version_allowlist, combined_allowlist, local_version_allowlist]
    else:
        return ["stubtest_allowlist.txt", platform_allowlist]


# Re-exposing as a public name to avoid many pyright reportPrivateUsage
TemporaryFileWrapper = tempfile._TemporaryFileWrapper  # pyright: ignore[reportPrivateUsage]

# We need to work around a limitation of tempfile.NamedTemporaryFile on Windows
# For details, see https://github.com/python/typeshed/pull/13620#discussion_r1990185997
# Python 3.12 added a cross-platform solution with `tempfile.NamedTemporaryFile("w+", delete_on_close=False)`
if sys.platform != "win32":
    NamedTemporaryFile = tempfile.NamedTemporaryFile  # noqa: TID251
else:

    def NamedTemporaryFile(mode: OpenTextMode) -> TemporaryFileWrapper[str]:  # noqa: N802
        def close(self: TemporaryFileWrapper[str]) -> None:
            TemporaryFileWrapper.close(self)  # pyright: ignore[reportUnknownMemberType]
            Path(self.name).unlink()

        temp = tempfile.NamedTemporaryFile(mode, delete=False)  # noqa: SIM115, TID251
        temp.close = MethodType(close, temp)  # type: ignore[method-assign]
        return temp


# ====================================================================
# Parsing .gitignore
# ====================================================================


@functools.cache
def get_gitignore_spec() -> pathspec.PathSpec:
    with GITIGNORE_PATH.open(encoding="UTF-8") as f:
        return pathspec.GitIgnoreSpec.from_lines(f)


def spec_matches_path(spec: pathspec.PathSpec, path: Path) -> bool:
    normalized_path = path.as_posix()
    if path.is_dir():
        normalized_path += "/"
    return spec.match_file(normalized_path)


# ====================================================================
# stubtest call
# ====================================================================


def allowlist_stubtest_arguments(distribution_name: str) -> list[str]:
    stubtest_arguments: list[str] = []
    for allowlist in allowlists(distribution_name):
        path = allowlists_path(distribution_name) / allowlist
        if path.exists():
            stubtest_arguments.extend(["--allowlist", str(path)])
    return stubtest_arguments



import re
from typing import ClassVar, Final

from docutils import nodes
from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class TitlePromoter(Transform):
    def promote_title(self, node: nodes.Element) -> bool: ...
    def promote_subtitle(self, node: nodes.Element) -> bool: ...
    def candidate_index(self, node: nodes.Element) -> tuple[nodes.Node, int] | tuple[None, None]: ...

class DocTitle(TitlePromoter):
    default_priority: ClassVar[int]
    def set_metadata(self) -> None: ...
    def apply(self) -> None: ...

class SectionSubTitle(TitlePromoter):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class DocInfo(Transform):
    default_priority: ClassVar[int]
    biblio_nodes: ClassVar[dict[str, type[nodes.Element]]]
    rcs_keyword_substitutions: ClassVar[list[tuple[re.Pattern[str], str]]]
    def apply(self) -> None: ...
    def extract_bibliographic(self, field_list): ...
    def check_empty_biblio_field(self, field, name) -> bool: ...
    def check_compound_biblio_field(self, field, name) -> bool: ...
    def extract_authors(self, field, name, docinfo) -> None: ...
    def authors_from_one_paragraph(self, field) -> list[list[nodes.Text]]: ...
    def authors_from_bullet_list(self, field): ...
    def authors_from_paragraphs(self, field): ...



from _typeshed import Incomplete
from typing import Any

from authlib.oauth1 import ClientAuth

class OAuth1Client:
    auth_class: type[ClientAuth] = ...
    session: Incomplete
    auth: Incomplete
    def __init__(
        self,
        session,
        client_id,
        client_secret=None,
        token=None,
        token_secret=None,
        redirect_uri=None,
        rsa_key=None,
        verifier=None,
        signature_method="HMAC-SHA1",
        signature_type="HEADER",
        force_include_body: bool = False,
        realm=None,
        **kwargs,
    ) -> None: ...
    @property
    def redirect_uri(self): ...
    @redirect_uri.setter
    def redirect_uri(self, uri) -> None: ...
    @property
    def token(self): ...
    @token.setter
    def token(self, token) -> None: ...
    def create_authorization_url(self, url, request_token=None, **kwargs): ...
    def fetch_request_token(self, url: str, **kwargs) -> dict[str, Any]: ...
    def fetch_access_token(self, url, verifier=None, **kwargs): ...
    def parse_authorization_response(self, url): ...
    def parse_response_token(self, status_code, text): ...
    @staticmethod
    def handle_error(error_type, error_description) -> None: ...
    def __del__(self) -> None: ...



import hashlib
from _typeshed import Incomplete

from authlib.jose.rfc7515 import JWSAlgorithm

class NoneAlgorithm(JWSAlgorithm):
    name: str
    description: str
    def prepare_key(self, raw_data) -> None: ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class HMACAlgorithm(JWSAlgorithm):
    SHA256 = hashlib.sha256
    SHA384 = hashlib.sha384
    SHA512 = hashlib.sha512
    name: Incomplete
    description: Incomplete
    hash_alg: Incomplete
    def __init__(self, sha_type) -> None: ...
    def prepare_key(self, raw_data): ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class RSAAlgorithm(JWSAlgorithm):
    SHA256: Incomplete
    SHA384: Incomplete
    SHA512: Incomplete
    name: Incomplete
    description: Incomplete
    hash_alg: Incomplete
    padding: Incomplete
    def __init__(self, sha_type) -> None: ...
    def prepare_key(self, raw_data): ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class ECAlgorithm(JWSAlgorithm):
    SHA256: Incomplete
    SHA384: Incomplete
    SHA512: Incomplete
    name: Incomplete
    curve: Incomplete
    description: Incomplete
    hash_alg: Incomplete
    def __init__(self, name, curve, sha_type) -> None: ...
    def prepare_key(self, raw_data): ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class RSAPSSAlgorithm(JWSAlgorithm):
    SHA256: Incomplete
    SHA384: Incomplete
    SHA512: Incomplete
    name: Incomplete
    description: Incomplete
    hash_alg: Incomplete
    def __init__(self, sha_type) -> None: ...
    def prepare_key(self, raw_data): ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

JWS_ALGORITHMS: list[JWSAlgorithm]



from collections.abc import Callable
from typing import Any, TypeVar
from typing_extensions import Self

import numpy as np
import tensorflow as tf
from tensorflow.core.example.example_pb2 import Example as Example
from tensorflow.core.example.feature_pb2 import (
    BytesList as BytesList,
    Feature as Feature,
    Features as Features,
    FloatList as FloatList,
    Int64List as Int64List,
)
from tensorflow.core.protobuf.cluster_pb2 import ClusterDef as ClusterDef
from tensorflow.core.protobuf.tensorflow_server_pb2 import ServerDef as ServerDef
from tensorflow.python.trackable.base import Trackable
from tensorflow.python.training.tracking.autotrackable import AutoTrackable

class CheckpointOptions:
    experimental_io_device: None | str
    experimental_enable_async_checkpoint: bool
    experimental_write_callbacks: None | list[Callable[[str], object] | Callable[[], object]]
    enable_async: bool

    def __init__(
        self,
        experimental_io_device: None | str = None,
        experimental_enable_async_checkpoint: bool = False,
        experimental_write_callbacks: None | list[Callable[[str], object] | Callable[[], object]] = None,
        enable_async: bool = False,
        experimental_skip_slot_variables: bool = False,
        experimental_sharding_callback=None,
    ) -> None: ...

_T = TypeVar("_T", bound=list[str] | tuple[str] | dict[int, str])

class ClusterSpec:
    def __init__(self, cluster: dict[str, _T] | ClusterDef | ClusterSpec) -> None: ...
    def as_dict(self) -> dict[str, list[str] | tuple[str] | dict[int, str]]: ...
    def num_tasks(self, job_name: str) -> int: ...

class _CheckpointLoadStatus:
    def assert_consumed(self) -> Self: ...
    def assert_existing_objects_matched(self) -> Self: ...
    def assert_nontrivial_match(self) -> Self: ...
    def expect_partial(self) -> Self: ...

class Checkpoint(AutoTrackable):
    def __init__(self, root: Trackable | None = None, **kwargs: Trackable) -> None: ...
    def read(self, save_path: str, options: CheckpointOptions | None = None) -> _CheckpointLoadStatus: ...
    def restore(self, save_path: str, options: CheckpointOptions | None = None) -> _CheckpointLoadStatus: ...
    def save(self, file_prefix: str, options: CheckpointOptions | None = None) -> str: ...
    def sync(self) -> None: ...
    def write(self, file_prefix: str, options: CheckpointOptions | None = None) -> str: ...

class CheckpointManager:
    def __init__(
        self,
        checkpoint: Checkpoint,
        directory: str,
        max_to_keep: int,
        keep_checkpoint_every_n_hours: int | None = None,
        checkpoint_name: str = "ckpt",
        step_counter: tf.Variable | None = None,
        checkpoint_interval: int | None = None,
        init_fn: Callable[[], object] | None = None,
    ) -> None: ...
    def _sweep(self) -> None: ...

def latest_checkpoint(checkpoint_dir: str, latest_filename: str | None = None) -> str: ...
def load_variable(ckpt_dir_or_file: str, name: str) -> np.ndarray[Any, Any]: ...
def list_variables(ckpt_dir_or_file: str) -> list[tuple[str, list[int]]]: ...
def __getattr__(name: str): ...  # incomplete module



from collections.abc import Sequence
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

@type_check_only
class _ProxyConfigDict(TypedDict):
    http: NotRequired[str]
    https: NotRequired[str]
    ftpProxy: NotRequired[str]
    noProxy: NotRequired[str]

@type_check_only
class _Environment(TypedDict):
    http_proxy: NotRequired[str]
    HTTP_PROXY: NotRequired[str]
    https_proxy: NotRequired[str]
    HTTPS_PROXY: NotRequired[str]
    ftp_proxy: NotRequired[str]
    FTP_PROXY: NotRequired[str]
    no_proxy: NotRequired[str]
    NO_PROXY: NotRequired[str]

class ProxyConfig(dict[str, str]):
    @property
    def http(self) -> str | None: ...
    @property
    def https(self) -> str | None: ...
    @property
    def ftp(self) -> str | None: ...
    @property
    def no_proxy(self) -> str | None: ...
    @staticmethod
    def from_dict(config: _ProxyConfigDict) -> ProxyConfig: ...
    def get_environment(self) -> _Environment: ...
    def inject_proxy_environment(self, environment: None | Sequence[str]) -> None | Sequence[str]: ...



from __future__ import annotations

import re
import sys
from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar

from attrs import NOTHING, Attribute

try:
    from inspect import get_annotations

    def get_annots(cl) -> dict[str, Any]:
        return get_annotations(cl, eval_str=True)

except ImportError:
    # https://docs.python.org/3/howto/annotations.html#accessing-the-annotations-dict-of-an-object-in-python-3-9-and-older
    def get_annots(cl) -> dict[str, Any]:
        if isinstance(cl, type):
            ann = cl.__dict__.get("__annotations__", {})
        else:
            ann = getattr(cl, "__annotations__", {})
        return ann


try:
    from typing_extensions import _TypedDictMeta
except ImportError:
    _TypedDictMeta = None

from .._compat import (
    TypedDict,
    get_full_type_hints,
    get_notrequired_base,
    get_origin,
    is_annotated,
    is_bare,
    is_generic,
)
from .._generics import deep_copy_with
from ..errors import (
    AttributeValidationNote,
    ClassValidationError,
    ForbiddenExtraKeysError,
    StructureHandlerNotFoundError,
)
from ..fns import identity
from . import AttributeOverride
from ._consts import already_generating, neutral
from ._generics import generate_mapping
from ._lc import generate_unique_filename
from ._shared import find_structure_handler

if TYPE_CHECKING:
    from ..converters import BaseConverter

__all__ = ["make_dict_unstructure_fn", "make_dict_structure_fn"]

T = TypeVar("T", bound=TypedDict)


def make_dict_unstructure_fn(
    cl: type[T],
    converter: BaseConverter,
    _cattrs_use_linecache: bool = True,
    **kwargs: AttributeOverride,
) -> Callable[[T], dict[str, Any]]:
    """
    Generate a specialized dict unstructuring function for a TypedDict.

    :param cl: A `TypedDict` class.
    :param converter: A Converter instance to use for unstructuring nested fields.
    :param kwargs: A mapping of field names to an `AttributeOverride`, for
        customization.
    :param _cattrs_detailed_validation: Whether to store the generated code in the
        _linecache_, for easier debugging and better stack traces.
    """
    origin = get_origin(cl)
    attrs = _adapted_fields(origin or cl)  # type: ignore
    req_keys = _required_keys(origin or cl)

    mapping = {}
    if is_generic(cl):
        mapping = generate_mapping(cl, mapping)

        for base in getattr(origin, "__orig_bases__", ()):
            if is_generic(base) and not str(base).startswith("typing.Generic"):
                mapping = generate_mapping(base, mapping)
                break

        # It's possible for origin to be None if this is a subclass
        # of a generic class.
        if origin is not None:
            cl = origin

    cl_name = cl.__name__
    fn_name = "unstructure_typeddict_" + cl_name
    globs = {}
    lines = []
    internal_arg_parts = {}

    # We keep track of what we're generating to help with recursive
    # class graphs.
    try:
        working_set = already_generating.working_set
    except AttributeError:
        working_set = set()
        already_generating.working_set = working_set
    if cl in working_set:
        raise RecursionError()
    working_set.add(cl)

    try:
        # We want to short-circuit in certain cases and return the identity
        # function.
        # We short-circuit if all of these are true:
        # * no attributes have been overridden
        # * all attributes resolve to `converter._unstructure_identity`
        for a in attrs:
            attr_name = a.name
            override = kwargs.get(attr_name, neutral)
            if override != neutral:
                break
            handler = None
            t = a.type

            if isinstance(t, TypeVar):
                if t.__name__ in mapping:
                    t = mapping[t.__name__]
                else:
                    # Unbound typevars use late binding.
                    handler = converter.unstructure
            elif is_generic(t) and not is_bare(t) and not is_annotated(t):
                t = deep_copy_with(t, mapping)

            if handler is None:
                nrb = get_notrequired_base(t)
                if nrb is not NOTHING:
                    t = nrb
                try:
                    handler = converter.get_unstructure_hook(t)
                except RecursionError:
                    # There's a circular reference somewhere down the line
                    handler = converter.unstructure
            is_identity = handler == identity
            if not is_identity:
                break
        else:
            # We've not broken the loop.
            return identity

        for ix, a in enumerate(attrs):
            attr_name = a.name
            override = kwargs.get(attr_name, neutral)
            if override.omit:
                lines.append(f"  res.pop('{attr_name}', None)")
                continue
            if override.rename is not None:
                # We also need to pop when renaming, since we're copying
                # the original.
                lines.append(f"  res.pop('{attr_name}', None)")
            kn = attr_name if override.rename is None else override.rename
            attr_required = attr_name in req_keys

            # For each attribute, we try resolving the type here and now.
            # If a type is manually overwritten, this function should be
            # regenerated.
            handler = None
            if override.unstruct_hook is not None:
                handler = override.unstruct_hook
            else:
                t = a.type

                if isinstance(t, TypeVar):
                    if t.__name__ in mapping:
                        t = mapping[t.__name__]
                    else:
                        handler = converter.unstructure
                elif is_generic(t) and not is_bare(t) and not is_annotated(t):
                    t = deep_copy_with(t, mapping)

                if handler is None:
                    nrb = get_notrequired_base(t)
                    if nrb is not NOTHING:
                        t = nrb
                    try:
                        handler = converter.get_unstructure_hook(t)
                    except RecursionError:
                        # There's a circular reference somewhere down the line
                        handler = converter.unstructure

            is_identity = handler == identity

            if not is_identity:
                unstruct_handler_name = f"__c_unstr_{ix}"
                globs[unstruct_handler_name] = handler
                internal_arg_parts[unstruct_handler_name] = handler
                invoke = f"{unstruct_handler_name}(instance['{attr_name}'])"
            elif override.rename is None:
                # We're not doing anything to this attribute, so
                # it'll already be present in the input dict.
                continue
            else:
                # Probably renamed, we just fetch it.
                invoke = f"instance['{attr_name}']"

            if attr_required:
                # No default or no override.
                lines.append(f"  res['{kn}'] = {invoke}")
            else:
                lines.append(f"  if '{attr_name}' in instance: res['{kn}'] = {invoke}")

        internal_arg_line = ", ".join([f"{i}={i}" for i in internal_arg_parts])
        if internal_arg_line:
            internal_arg_line = f", {internal_arg_line}"
        for k, v in internal_arg_parts.items():
            globs[k] = v

        total_lines = [
            f"def {fn_name}(instance{internal_arg_line}):",
            "  res = instance.copy()",
            *lines,
            "  return res",
        ]
        script = "\n".join(total_lines)

        fname = generate_unique_filename(
            cl, "unstructure", lines=total_lines if _cattrs_use_linecache else []
        )

        eval(compile(script, fname, "exec"), globs)
    finally:
        working_set.remove(cl)
        if not working_set:
            del already_generating.working_set

    return globs[fn_name]


def make_dict_structure_fn(
    cl: Any,
    converter: BaseConverter,
    _cattrs_forbid_extra_keys: bool | Literal["from_converter"] = "from_converter",
    _cattrs_use_linecache: bool = True,
    _cattrs_detailed_validation: bool | Literal["from_converter"] = "from_converter",
    **kwargs: AttributeOverride,
) -> Callable[[dict, Any], Any]:
    """Generate a specialized dict structuring function for typed dicts.

    :param cl: A `TypedDict` class.
    :param converter: A Converter instance to use for structuring nested fields.
    :param kwargs: A mapping of field names to an `AttributeOverride`, for
        customization.
    :param _cattrs_detailed_validation: Whether to use a slower mode that produces
        more detailed errors.
    :param _cattrs_forbid_extra_keys: Whether the structuring function should raise a
        `ForbiddenExtraKeysError` if unknown keys are encountered.
    :param _cattrs_detailed_validation: Whether to store the generated code in the
        _linecache_, for easier debugging and better stack traces.

    ..  versionchanged:: 23.2.0
        The `_cattrs_forbid_extra_keys` and `_cattrs_detailed_validation` parameters
        take their values from the given converter by default.
    """

    mapping = {}
    if is_generic(cl):
        base = get_origin(cl)
        mapping = generate_mapping(cl, mapping)
        if base is not None:
            # It's possible for this to be a subclass of a generic,
            # so no origin.
            cl = base

    for base in getattr(cl, "__orig_bases__", ()):
        if is_generic(base) and not str(base).startswith("typing.Generic"):
            mapping = generate_mapping(base, mapping)
            break

    cl_name = cl.__name__
    fn_name = "structure_" + cl_name

    # We have generic parameters and need to generate a unique name for the function
    for p in getattr(cl, "__parameters__", ()):
        try:
            name_base = mapping[p.__name__]
        except KeyError:
            pn = p.__name__
            raise StructureHandlerNotFoundError(
                f"Missing type for generic argument {pn}, specify it when structuring.",
                p,
            ) from None
        name = getattr(name_base, "__name__", None) or str(name_base)
        # `<>` can be present in lambdas
        # `|` can be present in unions
        name = re.sub(r"[\[\.\] ,<>]", "_", name)
        name = re.sub(r"\|", "u", name)
        fn_name += f"_{name}"

    internal_arg_parts = {"__cl": cl}
    globs = {}
    lines = []
    post_lines = []

    attrs = _adapted_fields(cl)
    req_keys = _required_keys(cl)

    allowed_fields = set()
    if _cattrs_forbid_extra_keys == "from_converter":
        # BaseConverter doesn't have it so we're careful.
        _cattrs_forbid_extra_keys = getattr(converter, "forbid_extra_keys", False)
    if _cattrs_detailed_validation == "from_converter":
        _cattrs_detailed_validation = converter.detailed_validation

    if _cattrs_forbid_extra_keys:
        globs["__c_a"] = allowed_fields
        globs["__c_feke"] = ForbiddenExtraKeysError

    lines.append("  res = o.copy()")

    if _cattrs_detailed_validation:
        lines.append("  errors = []")
        internal_arg_parts["__c_cve"] = ClassValidationError
        internal_arg_parts["__c_avn"] = AttributeValidationNote
        for ix, a in enumerate(attrs):
            an = a.name
            attr_required = an in req_keys
            override = kwargs.get(an, neutral)
            if override.omit:
                continue
            t = a.type

            if isinstance(t, TypeVar):
                t = mapping.get(t.__name__, t)
            elif is_generic(t) and not is_bare(t) and not is_annotated(t):
                t = deep_copy_with(t, mapping)

            nrb = get_notrequired_base(t)
            if nrb is not NOTHING:
                t = nrb

            if is_generic(t) and not is_bare(t) and not is_annotated(t):
                t = deep_copy_with(t, mapping)

            # For each attribute, we try resolving the type here and now.
            # If a type is manually overwritten, this function should be
            # regenerated.
            if override.struct_hook is not None:
                # If the user has requested an override, just use that.
                handler = override.struct_hook
            else:
                handler = find_structure_handler(a, t, converter)

            struct_handler_name = f"__c_structure_{ix}"
            internal_arg_parts[struct_handler_name] = handler

            kn = an if override.rename is None else override.rename
            allowed_fields.add(kn)
            i = "  "
            if not attr_required:
                lines.append(f"{i}if '{kn}' in o:")
                i = f"{i}  "
            lines.append(f"{i}try:")
            i = f"{i}  "

            tn = f"__c_type_{ix}"
            internal_arg_parts[tn] = t

            if handler == converter._structure_call:
                internal_arg_parts[struct_handler_name] = t
                lines.append(f"{i}res['{an}'] = {struct_handler_name}(o['{kn}'])")
            else:
                lines.append(f"{i}res['{an}'] = {struct_handler_name}(o['{kn}'], {tn})")
            if override.rename is not None:
                lines.append(f"{i}del res['{kn}']")
            i = i[:-2]
            lines.append(f"{i}except Exception as e:")
            i = f"{i}  "
            lines.append(
                f'{i}e.__notes__ = [*getattr(e, \'__notes__\', []), __c_avn("Structuring typeddict {cl.__qualname__} @ attribute {an}", "{an}", {tn})]'
            )
            lines.append(f"{i}errors.append(e)")

        if _cattrs_forbid_extra_keys:
            post_lines += [
                "  unknown_fields = o.keys() - __c_a",
                "  if unknown_fields:",
                "    errors.append(__c_feke('', __cl, unknown_fields))",
            ]

        post_lines.append(
            f"  if errors: raise __c_cve('While structuring ' + {cl.__name__!r}, errors, __cl)"
        )
    else:
        non_required = []

        # The first loop deals with required args.
        for ix, a in enumerate(attrs):
            an = a.name
            attr_required = an in req_keys
            override = kwargs.get(an, neutral)
            if override.omit:
                continue
            if not attr_required:
                non_required.append((ix, a))
                continue

            t = a.type

            if isinstance(t, TypeVar):
                t = mapping.get(t.__name__, t)
            elif is_generic(t) and not is_bare(t) and not is_annotated(t):
                t = deep_copy_with(t, mapping)

            nrb = get_notrequired_base(t)
            if nrb is not NOTHING:
                t = nrb

            if override.struct_hook is not None:
                handler = override.struct_hook
            else:
                # For each attribute, we try resolving the type here and now.
                # If a type is manually overwritten, this function should be
                # regenerated.
                handler = converter.get_structure_hook(t)

            kn = an if override.rename is None else override.rename
            allowed_fields.add(kn)

            struct_handler_name = f"__c_structure_{ix}"
            internal_arg_parts[struct_handler_name] = handler
            if handler == converter._structure_call:
                internal_arg_parts[struct_handler_name] = t
                invocation_line = f"  res['{an}'] = {struct_handler_name}(o['{kn}'])"
            else:
                tn = f"__c_type_{ix}"
                internal_arg_parts[tn] = t
                invocation_line = (
                    f"  res['{an}'] = {struct_handler_name}(o['{kn}'], {tn})"
                )

            lines.append(invocation_line)
            if override.rename is not None:
                lines.append(f"  del res['{override.rename}']")

        # The second loop is for optional args.
        if non_required:
            for ix, a in non_required:
                an = a.name
                override = kwargs.get(an, neutral)
                t = a.type

                nrb = get_notrequired_base(t)
                if nrb is not NOTHING:
                    t = nrb

                if isinstance(t, TypeVar):
                    t = mapping.get(t.__name__, t)
                elif is_generic(t) and not is_bare(t) and not is_annotated(t):
                    t = deep_copy_with(t, mapping)

                if override.struct_hook is not None:
                    handler = override.struct_hook
                else:
                    # For each attribute, we try resolving the type here and now.
                    # If a type is manually overwritten, this function should be
                    # regenerated.
                    handler = converter.get_structure_hook(t)

                struct_handler_name = f"__c_structure_{ix}"
                internal_arg_parts[struct_handler_name] = handler

                ian = an
                kn = an if override.rename is None else override.rename
                allowed_fields.add(kn)
                post_lines.append(f"  if '{kn}' in o:")
                if handler == converter._structure_call:
                    internal_arg_parts[struct_handler_name] = t
                    post_lines.append(
                        f"    res['{ian}'] = {struct_handler_name}(o['{kn}'])"
                    )
                else:
                    tn = f"__c_type_{ix}"
                    internal_arg_parts[tn] = t
                    post_lines.append(
                        f"    res['{ian}'] = {struct_handler_name}(o['{kn}'], {tn})"
                    )
                if override.rename is not None:
                    lines.append(f"  res.pop('{override.rename}', None)")

        if _cattrs_forbid_extra_keys:
            post_lines += [
                "  unknown_fields = o.keys() - __c_a",
                "  if unknown_fields:",
                "    raise __c_feke('', __cl, unknown_fields)",
            ]

    # At the end, we create the function header.
    internal_arg_line = ", ".join([f"{i}={i}" for i in internal_arg_parts])
    for k, v in internal_arg_parts.items():
        globs[k] = v

    total_lines = [
        f"def {fn_name}(o, _, {internal_arg_line}):",
        *lines,
        *post_lines,
        "  return res",
    ]

    script = "\n".join(total_lines)
    fname = generate_unique_filename(
        cl, "structure", lines=total_lines if _cattrs_use_linecache else []
    )

    eval(compile(script, fname, "exec"), globs)
    return globs[fn_name]


def _adapted_fields(cls: Any) -> list[Attribute]:
    annotations = get_annots(cls)
    hints = get_full_type_hints(cls)
    return [
        Attribute(
            n,
            NOTHING,
            None,
            False,
            False,
            False,
            False,
            False,
            type=hints[n] if n in hints else annotations[n],
        )
        for n, a in annotations.items()
    ]


def _is_extensions_typeddict(cls) -> bool:
    if _TypedDictMeta is None:
        return False
    return cls.__class__ is _TypedDictMeta or (
        is_generic(cls) and (cls.__origin__.__class__ is _TypedDictMeta)
    )


if sys.version_info >= (3, 11):

    def _required_keys(cls: type) -> set[str]:
        return cls.__required_keys__

elif sys.version_info >= (3, 9):
    from typing_extensions import Annotated, NotRequired, Required, get_args

    # Note that there is no `typing.Required` on 3.9 and 3.10, only in
    # `typing_extensions`. Therefore, `typing.TypedDict` will not honor this
    # annotation, only `typing_extensions.TypedDict`.

    def _required_keys(cls: type) -> set[str]:
        """Our own processor for required keys."""
        if _is_extensions_typeddict(cls):
            return cls.__required_keys__

        # We vendor a part of the typing_extensions logic for
        # gathering required keys. *sigh*
        own_annotations = cls.__dict__.get("__annotations__", {})
        required_keys = set()
        # On 3.8 - 3.10, typing.TypedDict doesn't put typeddict superclasses
        # in the MRO, therefore we cannot handle non-required keys properly
        # in some situations. Oh well.
        for key in getattr(cls, "__required_keys__", []):
            annotation_type = own_annotations[key]
            annotation_origin = get_origin(annotation_type)
            if annotation_origin is Annotated:
                annotation_args = get_args(annotation_type)
                if annotation_args:
                    annotation_type = annotation_args[0]
                    annotation_origin = get_origin(annotation_type)

            if annotation_origin is NotRequired:
                pass
            elif cls.__total__:
                required_keys.add(key)
        return required_keys

else:
    from typing_extensions import Annotated, NotRequired, Required, get_args

    # On 3.8, typing.TypedDicts do not have __required_keys__.

    def _required_keys(cls: type) -> set[str]:
        """Our own processor for required keys."""
        if _is_extensions_typeddict(cls):
            return cls.__required_keys__

        own_annotations = cls.__dict__.get("__annotations__", {})
        required_keys = set()
        for key in own_annotations:
            annotation_type = own_annotations[key]

            if is_annotated(annotation_type):
                # If this is `Annotated`, we need to get the origin twice.
                annotation_type = get_origin(annotation_type)

            annotation_origin = get_origin(annotation_type)

            if annotation_origin is Required:
                required_keys.add(key)
            elif annotation_origin is NotRequired:
                pass
            elif cls.__total__:
                required_keys.add(key)
        return required_keys



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["reciprocity", "overall_reciprocity"]

@_dispatchable
def reciprocity(G: Graph[_Node], nodes: Iterable[_Node] | None = None) -> float | dict[Incomplete, float | None]: ...
@_dispatchable
def overall_reciprocity(G: Graph[_Node]): ...



import sys
from _typeshed import Incomplete
from typing import Final

from networkx.utils.backends import _dispatchable

from ..classes.graph import Graph

if sys.version_info >= (3, 11):
    from importlib.resources.abc import Traversable
else:
    from importlib.abc import Traversable

__all__ = ["graph_atlas", "graph_atlas_g"]

NUM_GRAPHS: Final = 1253
ATLAS_FILE: Final[Traversable]

@_dispatchable
def graph_atlas(i) -> Graph[Incomplete]: ...
@_dispatchable
def graph_atlas_g() -> list[Graph[Incomplete]]: ...



from collections.abc import Iterable
from typing import Literal, SupportsFloat, SupportsIndex
from typing_extensions import Self, TypeAlias

from .._typing import ArrayLikeSeq
from ..constructive import BufferJoinStyle
from .base import BaseGeometry
from .multilinestring import MultiLineString
from .multipoint import MultiPoint
from .point import Point
from .polygon import Polygon

__all__ = ["LineString"]

_ConvertibleToLineString: TypeAlias = LineString | ArrayLikeSeq[float] | Iterable[Point | Iterable[SupportsFloat]]

class LineString(BaseGeometry):
    __slots__: list[str] = []
    def __new__(self, coordinates: _ConvertibleToLineString | None = None) -> Self: ...
    def svg(self, scale_factor: float = 1.0, stroke_color: str | None = None, opacity: float | None = None) -> str: ...  # type: ignore[override]
    def offset_curve(
        self,
        distance: float,
        quad_segs: SupportsIndex = 16,
        join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = ...,
        mitre_limit: float = 5.0,
    ) -> LineString | MultiLineString: ...
    def parallel_offset(  # to be deprecated
        self,
        distance: float,
        side: str = "right",
        resolution: SupportsIndex = 16,
        join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = ...,
        mitre_limit: float = 5.0,
    ) -> LineString | MultiLineString: ...
    # more precise base overrides
    @property
    def boundary(self) -> MultiPoint: ...
    @property
    def convex_hull(self) -> LineString: ...
    @property
    def envelope(self) -> Polygon: ...
    @property
    def oriented_envelope(self) -> LineString: ...
    @property
    def minimum_rotated_rectangle(self) -> LineString: ...



from builtins import bytes as bytes, str as str
from collections import OrderedDict as OrderedDict
from typing import Literal
from typing_extensions import TypeAlias
from urllib.parse import (
    quote as quote,
    quote_plus as quote_plus,
    unquote as unquote,
    unquote_plus as unquote_plus,
    urldefrag as urldefrag,
    urlencode as urlencode,
    urljoin as urljoin,
    urlparse as urlparse,
    urlsplit as urlsplit,
    urlunparse as urlunparse,
)
from urllib.request import getproxies as getproxies, parse_http_list as parse_http_list, proxy_bypass as proxy_bypass

is_urllib3_1: bool
is_py2: Literal[False]
is_py3: Literal[True]
has_simplejson: bool

builtin_str: TypeAlias = str  # noqa: Y042
basestring: tuple[type, ...]
numeric_types: tuple[type, ...]
integer_types: tuple[type, ...]



import io
import logging
import re
import socket
from _typeshed import ReadableBuffer
from collections.abc import Callable
from typing import Any

from gunicorn.config import Config
from gunicorn.http import Request

from .._types import _AddressType, _EnvironType, _HeadersType, _StatusType

BLKSIZE: int
HEADER_VALUE_RE: re.Pattern[str]
log: logging.Logger

class FileWrapper:
    filelike: io.IOBase
    blksize: int
    close: Callable[[], None] | None

    def __init__(self, filelike: io.IOBase, blksize: int = 8192) -> None: ...
    def __getitem__(self, key: Any) -> bytes: ...

class WSGIErrorsWrapper(io.RawIOBase):
    streams: list[io.TextIOBase]

    def __init__(self, cfg: Config) -> None: ...
    def write(self, data: ReadableBuffer) -> None: ...

def base_environ(cfg: Config) -> _EnvironType: ...
def default_environ(req: Request, sock: socket.socket, cfg: Config) -> _EnvironType: ...
def proxy_environ(req: Request) -> _EnvironType: ...
def create(
    req: Request, sock: socket.socket, client: _AddressType, server: _AddressType, cfg: Config
) -> tuple[Response, _EnvironType]: ...

class Response:
    req: Request
    sock: socket.socket
    version: str
    status: str | None
    chunked: bool
    must_close: bool
    headers: _HeadersType
    headers_sent: bool
    response_length: int | None
    sent: int
    upgrade: bool
    cfg: Config
    status_code: int | None

    def __init__(self, req: Request, sock: socket.socket, cfg: Config) -> None: ...
    def force_close(self) -> None: ...
    def should_close(self) -> bool: ...
    def start_response(
        self, status: _StatusType, headers: _HeadersType, exc_info: tuple[type, BaseException, Any] | None = None
    ) -> Callable[[bytes], None]: ...
    def process_headers(self, headers: _HeadersType) -> None: ...
    def is_chunked(self) -> bool: ...
    def default_headers(self) -> list[str]: ...
    def send_headers(self) -> None: ...
    def write(self, arg: bytes) -> None: ...
    def can_sendfile(self) -> bool: ...
    def sendfile(self, respiter: FileWrapper) -> bool: ...
    def write_file(self, respiter: FileWrapper) -> None: ...
    def close(self) -> None: ...



from .menu_borders import (
    AsciiBorderStyle as AsciiBorderStyle,
    DoubleLineBorderStyle as DoubleLineBorderStyle,
    DoubleLineOuterLightInnerBorderStyle as DoubleLineOuterLightInnerBorderStyle,
    HeavyBorderStyle as HeavyBorderStyle,
    HeavyOuterLightInnerBorderStyle as HeavyOuterLightInnerBorderStyle,
    LightBorderStyle as LightBorderStyle,
    MenuBorderStyle as MenuBorderStyle,
    MenuBorderStyleFactory as MenuBorderStyleFactory,
    MenuBorderStyleType as MenuBorderStyleType,
)
from .menu_margins import MenuMargins as MenuMargins
from .menu_padding import MenuPadding as MenuPadding
from .menu_style import MenuStyle as MenuStyle

__all__ = [
    "MenuBorderStyle",
    "MenuBorderStyleType",
    "MenuBorderStyleFactory",
    "MenuMargins",
    "MenuPadding",
    "MenuStyle",
    "AsciiBorderStyle",
    "LightBorderStyle",
    "HeavyBorderStyle",
    "DoubleLineBorderStyle",
    "DoubleLineOuterLightInnerBorderStyle",
    "HeavyOuterLightInnerBorderStyle",
]



from _typeshed import Unused
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

from .sources import Source

@type_check_only
class _Instruction(TypedDict):
    address: str
    instruction: str
    instructionBytes: str
    symbol: NotRequired[str]  # only set if there's a corresponding label
    line: NotRequired[int]  # only set if source is available
    location: NotRequired[Source]  # only set if source is available

@type_check_only
class _DisassembleResult(TypedDict):
    instructions: list[_Instruction]

def disassemble(
    *, memoryReference: str, offset: int = 0, instructionOffset: int = 0, instructionCount: int, **extra: Unused
) -> _DisassembleResult: ...



import _ast
import sys
import types
from _typeshed import ReadableBuffer, StrPath
from abc import ABCMeta, abstractmethod
from collections.abc import Iterator, Mapping, Sequence
from importlib import _bootstrap_external
from importlib.machinery import ModuleSpec
from io import BufferedReader
from typing import IO, Any, Literal, Protocol, overload, runtime_checkable
from typing_extensions import deprecated

if sys.version_info >= (3, 11):
    __all__ = [
        "Loader",
        "MetaPathFinder",
        "PathEntryFinder",
        "ResourceLoader",
        "InspectLoader",
        "ExecutionLoader",
        "FileLoader",
        "SourceLoader",
    ]

    if sys.version_info < (3, 12):
        __all__ += ["Finder", "ResourceReader", "Traversable", "TraversableResources"]

if sys.version_info >= (3, 10):
    from importlib._abc import Loader as Loader
else:
    class Loader(metaclass=ABCMeta):
        def load_module(self, fullname: str) -> types.ModuleType: ...
        def module_repr(self, module: types.ModuleType) -> str: ...
        def create_module(self, spec: ModuleSpec) -> types.ModuleType | None: ...
        # Not defined on the actual class for backwards-compatibility reasons,
        # but expected in new code.
        def exec_module(self, module: types.ModuleType) -> None: ...

if sys.version_info < (3, 12):
    @deprecated("Deprecated since Python 3.3; removed in Python 3.12. Use `MetaPathFinder` or `PathEntryFinder` instead.")
    class Finder(metaclass=ABCMeta): ...

@deprecated("Deprecated since Python 3.7. Use `importlib.resources.abc.TraversableResources` instead.")
class ResourceLoader(Loader):
    @abstractmethod
    def get_data(self, path: str) -> bytes: ...

class InspectLoader(Loader):
    def is_package(self, fullname: str) -> bool: ...
    def get_code(self, fullname: str) -> types.CodeType | None: ...
    @abstractmethod
    def get_source(self, fullname: str) -> str | None: ...
    def exec_module(self, module: types.ModuleType) -> None: ...
    @staticmethod
    def source_to_code(
        data: ReadableBuffer | str | _ast.Module | _ast.Expression | _ast.Interactive, path: bytes | StrPath = "<string>"
    ) -> types.CodeType: ...

class ExecutionLoader(InspectLoader):
    @abstractmethod
    def get_filename(self, fullname: str) -> str: ...

class SourceLoader(_bootstrap_external.SourceLoader, ResourceLoader, ExecutionLoader, metaclass=ABCMeta):  # type: ignore[misc]  # incompatible definitions of source_to_code in the base classes
    @deprecated("Deprecated since Python 3.3. Use `importlib.resources.abc.SourceLoader.path_stats` instead.")
    def path_mtime(self, path: str) -> float: ...
    def set_data(self, path: str, data: bytes) -> None: ...
    def get_source(self, fullname: str) -> str | None: ...
    def path_stats(self, path: str) -> Mapping[str, Any]: ...

# The base classes differ starting in 3.10:
if sys.version_info >= (3, 10):
    # Please keep in sync with _typeshed.importlib.MetaPathFinderProtocol
    class MetaPathFinder(metaclass=ABCMeta):
        if sys.version_info < (3, 12):
            @deprecated("Deprecated since Python 3.4; removed in Python 3.12. Use `MetaPathFinder.find_spec()` instead.")
            def find_module(self, fullname: str, path: Sequence[str] | None) -> Loader | None: ...

        def invalidate_caches(self) -> None: ...
        # Not defined on the actual class, but expected to exist.
        def find_spec(
            self, fullname: str, path: Sequence[str] | None, target: types.ModuleType | None = ..., /
        ) -> ModuleSpec | None: ...

    class PathEntryFinder(metaclass=ABCMeta):
        if sys.version_info < (3, 12):
            @deprecated("Deprecated since Python 3.4; removed in Python 3.12. Use `PathEntryFinder.find_spec()` instead.")
            def find_module(self, fullname: str) -> Loader | None: ...
            @deprecated("Deprecated since Python 3.4; removed in Python 3.12. Use `find_spec()` instead.")
            def find_loader(self, fullname: str) -> tuple[Loader | None, Sequence[str]]: ...

        def invalidate_caches(self) -> None: ...
        # Not defined on the actual class, but expected to exist.
        def find_spec(self, fullname: str, target: types.ModuleType | None = ...) -> ModuleSpec | None: ...

else:
    # Please keep in sync with _typeshed.importlib.MetaPathFinderProtocol
    class MetaPathFinder(Finder):
        def find_module(self, fullname: str, path: Sequence[str] | None) -> Loader | None: ...
        def invalidate_caches(self) -> None: ...
        # Not defined on the actual class, but expected to exist.
        def find_spec(
            self, fullname: str, path: Sequence[str] | None, target: types.ModuleType | None = ..., /
        ) -> ModuleSpec | None: ...

    class PathEntryFinder(Finder):
        def find_module(self, fullname: str) -> Loader | None: ...
        def find_loader(self, fullname: str) -> tuple[Loader | None, Sequence[str]]: ...
        def invalidate_caches(self) -> None: ...
        # Not defined on the actual class, but expected to exist.
        def find_spec(self, fullname: str, target: types.ModuleType | None = ...) -> ModuleSpec | None: ...

class FileLoader(_bootstrap_external.FileLoader, ResourceLoader, ExecutionLoader, metaclass=ABCMeta):
    name: str
    path: str
    def __init__(self, fullname: str, path: str) -> None: ...
    def get_data(self, path: str) -> bytes: ...
    def get_filename(self, fullname: str | None = None) -> str: ...
    def load_module(self, fullname: str | None = None) -> types.ModuleType: ...

if sys.version_info < (3, 11):
    class ResourceReader(metaclass=ABCMeta):
        @abstractmethod
        def open_resource(self, resource: str) -> IO[bytes]: ...
        @abstractmethod
        def resource_path(self, resource: str) -> str: ...
        if sys.version_info >= (3, 10):
            @abstractmethod
            def is_resource(self, path: str) -> bool: ...
        else:
            @abstractmethod
            def is_resource(self, name: str) -> bool: ...

        @abstractmethod
        def contents(self) -> Iterator[str]: ...

    @runtime_checkable
    class Traversable(Protocol):
        @abstractmethod
        def is_dir(self) -> bool: ...
        @abstractmethod
        def is_file(self) -> bool: ...
        @abstractmethod
        def iterdir(self) -> Iterator[Traversable]: ...
        if sys.version_info >= (3, 11):
            @abstractmethod
            def joinpath(self, *descendants: str) -> Traversable: ...
        else:
            @abstractmethod
            def joinpath(self, child: str, /) -> Traversable: ...

        # The documentation and runtime protocol allows *args, **kwargs arguments,
        # but this would mean that all implementers would have to support them,
        # which is not the case.
        @overload
        @abstractmethod
        def open(self, mode: Literal["r"] = "r", *, encoding: str | None = None, errors: str | None = None) -> IO[str]: ...
        @overload
        @abstractmethod
        def open(self, mode: Literal["rb"]) -> IO[bytes]: ...
        @property
        @abstractmethod
        def name(self) -> str: ...
        if sys.version_info >= (3, 10):
            def __truediv__(self, child: str, /) -> Traversable: ...
        else:
            @abstractmethod
            def __truediv__(self, child: str, /) -> Traversable: ...

        @abstractmethod
        def read_bytes(self) -> bytes: ...
        @abstractmethod
        def read_text(self, encoding: str | None = None) -> str: ...

    class TraversableResources(ResourceReader):
        @abstractmethod
        def files(self) -> Traversable: ...
        def open_resource(self, resource: str) -> BufferedReader: ...
        def resource_path(self, resource: Any) -> str: ...
        def is_resource(self, path: str) -> bool: ...
        def contents(self) -> Iterator[str]: ...

elif sys.version_info < (3, 14):
    from importlib.resources.abc import (
        ResourceReader as ResourceReader,
        Traversable as Traversable,
        TraversableResources as TraversableResources,
    )



from typing_extensions import Buffer

from gunicorn.http import Message

class ParseException(Exception): ...

class NoMoreData(IOError):
    buf: Buffer

    def __init__(self, buf: Buffer | None = None) -> None: ...

class ConfigurationProblem(ParseException):
    info: str
    code: int

    def __init__(self, info: str) -> None: ...

class InvalidRequestLine(ParseException):
    req: str
    code: int

    def __init__(self, req: str) -> None: ...

class InvalidRequestMethod(ParseException):
    method: str

    def __init__(self, method: str) -> None: ...

class InvalidHTTPVersion(ParseException):
    version: str | tuple[int, int]

    def __init__(self, version: str | tuple[int, int]) -> None: ...

class InvalidHeader(ParseException):
    hdr: str
    req: Message | None

    def __init__(self, hdr: str, req: Message | None = None) -> None: ...

class ObsoleteFolding(ParseException):
    hdr: str

    def __init__(self, hdr: str) -> None: ...

class InvalidHeaderName(ParseException):
    hdr: str

    def __init__(self, hdr: str) -> None: ...

class UnsupportedTransferCoding(ParseException):
    hdr: str
    code: int

    def __init__(self, hdr: str) -> None: ...

class InvalidChunkSize(IOError):
    data: bytes

    def __init__(self, data: bytes) -> None: ...

class ChunkMissingTerminator(IOError):
    term: bytes

    def __init__(self, term: bytes) -> None: ...

class LimitRequestLine(ParseException):
    size: int
    max_size: int

    def __init__(self, size: int, max_size: int) -> None: ...

class LimitRequestHeaders(ParseException):
    msg: str

    def __init__(self, msg: str) -> None: ...

class InvalidProxyLine(ParseException):
    line: str
    code: int

    def __init__(self, line: str) -> None: ...

class ForbiddenProxyRequest(ParseException):
    host: str
    code: int

    def __init__(self, host: str) -> None: ...

class InvalidSchemeHeaders(ParseException): ...



from .claims import ClientMetadataClaims as ClientMetadataClaims
from .endpoint import ClientRegistrationEndpoint as ClientRegistrationEndpoint
from .errors import (
    InvalidClientMetadataError as InvalidClientMetadataError,
    InvalidRedirectURIError as InvalidRedirectURIError,
    InvalidSoftwareStatementError as InvalidSoftwareStatementError,
    UnapprovedSoftwareStatementError as UnapprovedSoftwareStatementError,
)

__all__ = [
    "ClientMetadataClaims",
    "ClientRegistrationEndpoint",
    "InvalidRedirectURIError",
    "InvalidClientMetadataError",
    "InvalidSoftwareStatementError",
    "UnapprovedSoftwareStatementError",
]



from __future__ import annotations

from typing_extensions import assert_type

import psycopg2
from psycopg2.extensions import connection, cursor


class MyCursor(cursor):
    pass


class MyConnection(connection):
    pass


def custom_connection(dsn: str) -> MyConnection:
    return MyConnection(dsn, async_=0)


# -> psycopg2.extensions.connection
assert_type(psycopg2.connect(), connection)
assert_type(psycopg2.connect("test-conn"), connection)
assert_type(psycopg2.connect(None), connection)
assert_type(psycopg2.connect("test-conn", connection_factory=None), connection)

assert_type(psycopg2.connect(cursor_factory=MyCursor), connection)
assert_type(psycopg2.connect("test-conn", cursor_factory=MyCursor), connection)
assert_type(psycopg2.connect(None, cursor_factory=MyCursor), connection)
assert_type(psycopg2.connect("test-conn", connection_factory=None, cursor_factory=MyCursor), connection)

# -> custom_connection
assert_type(psycopg2.connect(connection_factory=MyConnection), MyConnection)
assert_type(psycopg2.connect("test-conn", connection_factory=MyConnection), MyConnection)
assert_type(psycopg2.connect("test-conn", MyConnection), MyConnection)
assert_type(psycopg2.connect(connection_factory=custom_connection), MyConnection)

assert_type(psycopg2.connect(connection_factory=MyConnection, cursor_factory=MyCursor), MyConnection)
assert_type(psycopg2.connect("test-conn", connection_factory=MyConnection, cursor_factory=MyCursor), MyConnection)
assert_type(psycopg2.connect(connection_factory=custom_connection, cursor_factory=MyCursor), MyConnection)



from networkx.algorithms.minors.contraction import (
    contracted_edge as contracted_edge,
    contracted_nodes as contracted_nodes,
    equivalence_classes as equivalence_classes,
    identified_nodes as identified_nodes,
    quotient_graph as quotient_graph,
)

__all__ = ["contracted_edge", "contracted_nodes", "equivalence_classes", "identified_nodes", "quotient_graph"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class TPUHardwareFeature(google.protobuf.message.Message):
    """Describes features of a tpu."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _EmbeddingFeature:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _EmbeddingFeatureEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[TPUHardwareFeature._EmbeddingFeature.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSUPPORTED: TPUHardwareFeature._EmbeddingFeature.ValueType  # 0
        """No embedding lookup accelerator available on the tpu."""
        V1: TPUHardwareFeature._EmbeddingFeature.ValueType  # 1
        """Embedding lookup accelerator V1. The embedding lookup operation can only
        be placed at the beginning of computation. Only one instance of embedding
        lookup layer is allowed.
        """
        V2: TPUHardwareFeature._EmbeddingFeature.ValueType  # 2
        """Embedding lookup accelerator V2. The embedding lookup operation can be
        placed anywhere of the computation. Multiple instances of embedding
        lookup layer is allowed.
        """

    class EmbeddingFeature(_EmbeddingFeature, metaclass=_EmbeddingFeatureEnumTypeWrapper):
        """Embedding feature of a tpu."""

    UNSUPPORTED: TPUHardwareFeature.EmbeddingFeature.ValueType  # 0
    """No embedding lookup accelerator available on the tpu."""
    V1: TPUHardwareFeature.EmbeddingFeature.ValueType  # 1
    """Embedding lookup accelerator V1. The embedding lookup operation can only
    be placed at the beginning of computation. Only one instance of embedding
    lookup layer is allowed.
    """
    V2: TPUHardwareFeature.EmbeddingFeature.ValueType  # 2
    """Embedding lookup accelerator V2. The embedding lookup operation can be
    placed anywhere of the computation. Multiple instances of embedding
    lookup layer is allowed.
    """

    EMBEDDING_FEATURE_FIELD_NUMBER: builtins.int
    NUM_EMBEDDING_DEVICES_PER_CHIP_FIELD_NUMBER: builtins.int
    embedding_feature: global___TPUHardwareFeature.EmbeddingFeature.ValueType
    num_embedding_devices_per_chip: builtins.int
    """Number of embedding accelerator devices per chip."""
    def __init__(
        self,
        *,
        embedding_feature: global___TPUHardwareFeature.EmbeddingFeature.ValueType | None = ...,
        num_embedding_devices_per_chip: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "embedding_feature", b"embedding_feature", "num_embedding_devices_per_chip", b"num_embedding_devices_per_chip"
        ],
    ) -> None: ...

global___TPUHardwareFeature = TPUHardwareFeature

@typing.final
class TopologyProto(google.protobuf.message.Message):
    """Describes the geometry of a TPU mesh."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESH_SHAPE_FIELD_NUMBER: builtins.int
    NUM_TASKS_FIELD_NUMBER: builtins.int
    NUM_TPU_DEVICES_PER_TASK_FIELD_NUMBER: builtins.int
    DEVICE_COORDINATES_FIELD_NUMBER: builtins.int
    TPU_HARDWARE_FEATURE_FIELD_NUMBER: builtins.int
    num_tasks: builtins.int
    """Number of TensorFlow tasks in the cluster."""
    num_tpu_devices_per_task: builtins.int
    """Number of TPU devices per task."""
    @property
    def mesh_shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """The dimensions of the TPU topology, in cores. Typically, this is a 4D
        topology [x, y, z, core], where the major dimensions correspond to TPU
        chips, and the minor dimension describes the number of cores on a multicore
        chip.
        """

    @property
    def device_coordinates(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """A flattened rank 3 int32 array with shape
        [num_tasks, num_tpu_devices_per_task, len(mesh_shape)].
        `tasks` is the number of tasks in the TPU cluster, `devices` is the number
        of TPU devices per task, and the minor dimension corresponds to a position
        in the TPU mesh topology. Each entry [task, device, axis] gives the
        `axis`-th coordinate in the topology of a task/device pair.
        """

    @property
    def tpu_hardware_feature(self) -> global___TPUHardwareFeature:
        """TPU supported features."""

    def __init__(
        self,
        *,
        mesh_shape: collections.abc.Iterable[builtins.int] | None = ...,
        num_tasks: builtins.int | None = ...,
        num_tpu_devices_per_task: builtins.int | None = ...,
        device_coordinates: collections.abc.Iterable[builtins.int] | None = ...,
        tpu_hardware_feature: global___TPUHardwareFeature | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["tpu_hardware_feature", b"tpu_hardware_feature"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "device_coordinates",
            b"device_coordinates",
            "mesh_shape",
            b"mesh_shape",
            "num_tasks",
            b"num_tasks",
            "num_tpu_devices_per_task",
            b"num_tpu_devices_per_task",
            "tpu_hardware_feature",
            b"tpu_hardware_feature",
        ],
    ) -> None: ...

global___TopologyProto = TopologyProto



from typing import Final

class PaymentInstrumentType:
    AmexExpressCheckoutCard: Final = "amex_express_checkout_card"
    AndroidPayCard: Final = "android_pay_card"
    ApplePayCard: Final = "apple_pay_card"
    CreditCard: Final = "credit_card"
    EuropeBankAccount: Final = "europe_bank_account"
    LocalPayment: Final = "local_payment"
    MasterpassCard: Final = "masterpass_card"
    MetaCheckoutCard: Final = "meta_checkout_card"
    MetaCheckoutToken: Final = "meta_checkout_token"
    PayPalAccount: Final = "paypal_account"
    PayPalHere: Final = "paypal_here"
    SamsungPayCard: Final = "samsung_pay_card"
    SepaDirectDebitAccount: Final = "sepa_debit_account"
    UsBankAccount: Final = "us_bank_account"
    VenmoAccount: Final = "venmo_account"
    VisaCheckoutCard: Final = "visa_checkout_card"



__all__ = ["JSONDecodeError"]

def linecol(doc: str, pos: int) -> tuple[int, int]: ...
def errmsg(msg: str, doc: str, pos: int, end: int | None = None) -> str: ...

class JSONDecodeError(ValueError):
    msg: str
    doc: str
    pos: int
    end: int | None
    lineno: int
    colno: int
    endlineno: int | None
    endcolno: int | None
    def __init__(self, msg: str, doc: str, pos: int, end: int | None = None) -> None: ...
    def __reduce__(self) -> tuple[JSONDecodeError, tuple[str, str, int, int | None]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import google.protobuf.descriptor
from tensorflow.tsl.protobuf.rpc_options_pb2 import RPCOptions as RPCOptions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["degree_centrality", "betweenness_centrality", "closeness_centrality"]

@_dispatchable
def degree_centrality(G: Graph[_Node], nodes) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def betweenness_centrality(G: Graph[_Node], nodes) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def closeness_centrality(G: Graph[_Node], nodes, normalized: bool | None = True) -> dict[Incomplete, Incomplete]: ...



from __future__ import annotations

from ctypes import c_char, c_float, c_int
from multiprocessing import Array, Value
from multiprocessing.sharedctypes import Synchronized, SynchronizedArray, SynchronizedString
from typing_extensions import assert_type

string = Array(c_char, 12)
assert_type(string, SynchronizedString)
assert_type(string.value, bytes)

numbers = Array(c_int, 3)
assert_type(numbers, SynchronizedArray[int])
numbers[0] = 3
numbers[:] = [0, 1, 2]

field = Value(c_float, 0.0)
assert_type(field, Synchronized[float])
field.value = 1.2



from typing import Final

from .core import Shell as Shell, make_click_shell as make_click_shell
from .decorators import shell as shell

__all__ = ["make_click_shell", "shell", "Shell", "__version__"]
__version__: Final[str]



from lib2to3 import fixer_base
from typing import ClassVar, Literal

class FixReduce(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    order: ClassVar[Literal["pre"]]
    PATTERN: ClassVar[str]
    def transform(self, node, results) -> None: ...



import re
from _typeshed import StrPath
from typing import ClassVar, Final

from docutils import nodes
from docutils.writers import html4css1

__docformat__: Final = "reStructuredText"
themes_dir_path: Final[str]

def find_theme(name: StrPath) -> str: ...

class Writer(html4css1.Writer):
    settings_default_overrides: ClassVar[dict[str, int]]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    translator_class: type[S5HTMLTranslator]

class S5HTMLTranslator(html4css1.HTMLTranslator):
    s5_stylesheet_template: ClassVar[str]
    disable_current_slide: ClassVar[str]
    layout_template: ClassVar[str]
    default_theme: ClassVar[str]
    base_theme_file: ClassVar[str]
    direct_theme_files: ClassVar[tuple[str, ...]]
    indirect_theme_files: ClassVar[tuple[str, ...]]
    required_theme_files: ClassVar[tuple[str, ...]]
    theme_file_path: str | None
    s5_footer: list[str]
    s5_header: list[str]
    section_count: int
    theme_files_copied: dict[str, bool]
    def __init__(self, document: nodes.document, /) -> None: ...
    def setup_theme(self) -> None: ...
    def copy_theme(self) -> None: ...
    files_to_skip_pattern: re.Pattern[str]
    def copy_file(self, name, source_dir, dest_dir): ...
    def depart_document(self, node: nodes.document) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_section(self, node: nodes.section) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_title(self, node: nodes.title) -> None: ...



from _typeshed import Incomplete
from typing import Final

from networkx import DiGraph
from networkx.utils.backends import _dispatchable

__all__ = ["triad_graph"]

TRIAD_EDGES: Final[dict[str, list[str]]]

@_dispatchable
def triad_graph(triad_name) -> DiGraph[Incomplete]: ...



from _typeshed import Incomplete, SupportsRead, Unused
from collections.abc import Callable, Iterable, Sequence
from typing import Any, Literal, NoReturn, Protocol, type_check_only
from typing_extensions import Self, TypeAlias

from reportlab.lib.colors import Color
from reportlab.lib.styles import ListStyle, ParagraphStyle, PropertySet
from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfgen.textobject import _Color
from reportlab.platypus.paragraph import Paragraph

__all__ = [
    "AnchorFlowable",
    "BalancedColumns",
    "BulletDrawer",
    "CallerMacro",
    "CondPageBreak",
    "DDIndenter",
    "DocAssert",
    "DocAssign",
    "DocExec",
    "DocIf",
    "DocPara",
    "DocWhile",
    "FailOnDraw",
    "FailOnWrap",
    "Flowable",
    "FrameBG",
    "FrameSplitter",
    "HRFlowable",
    "Image",
    "ImageAndFlowables",
    "KeepInFrame",
    "KeepTogether",
    "LIIndenter",
    "ListFlowable",
    "ListItem",
    "Macro",
    "NullDraw",
    "PTOContainer",
    "PageBreak",
    "PageBreakIfNotEmpty",
    "ParagraphAndImage",
    "Preformatted",
    "SetPageTopFlowables",
    "SetTopFlowables",
    "SlowPageBreak",
    "Spacer",
    "TopPadder",
    "TraceInfo",
    "UseUpSpace",
    "XBox",
    "splitLine",
    "splitLines",
    "PlacedStory",
]

_HAlignment: TypeAlias = Literal["LEFT", "CENTER", "CENTRE", "RIGHT", 0, 1, 2]
_VAlignment: TypeAlias = Literal["BOTTOM", "MIDDLE", "TOP"]
# FIXME: Consider using Sequence[Flowable] for covariance on list, even though
#        that will give false negatives for non list or tuple sequences, it also
#        would reduce type safety, since flowables don't copy the list
_FlowableSublist: TypeAlias = Flowable | list[Flowable] | tuple[Flowable, ...]
# NOTE: Technically can only be list or tuple, but would be annoying for variance
_NestedFlowable: TypeAlias = Flowable | Sequence[_NestedFlowable]

@type_check_only
class _StyledFlowableFactory(Protocol):
    # NOTE: We leave style at Any so people can specify a specifc property set
    def __call__(self, value: str, /, *, style: Any) -> Flowable: ...

class TraceInfo:
    srcFile: str
    startLineNo: int
    startLinePos: int
    endLineNo: int
    endLinePos: int
    def __init__(self) -> None: ...

class Flowable:
    width: float
    height: float
    wrapped: int
    hAlign: _HAlignment
    vAlign: _VAlignment
    encoding: str | None
    # NOTE: this only exists during drawing, splitting and wrapping
    canv: Canvas
    # NOTE: The following attributes will not exist on all flowables, but
    #       they need to be settable on individual instances
    keepWithNext: Incomplete
    spaceAfter: float
    spaceBefore: float
    def __init__(self) -> None: ...
    # NOTE: We pretend the optional internal _sW argument does not exist
    #       since not all flowables support it and we'd have to deal with
    #       a bunch of LSP errors. Conversely we will get type errors in
    #       subclasses that rely on the argument existing when called through
    #       super() inside their own implementation, so we can't really
    #       make everyone happy here, sigh...
    def drawOn(self, canvas: Canvas, x: float, y: float) -> None: ...
    def wrapOn(self, canv: Canvas, aW: float, aH: float) -> tuple[float, float]: ...
    def wrap(self, aW: float, aH: float) -> tuple[float, float]: ...
    def minWidth(self) -> float: ...
    def splitOn(self, canv: Canvas, aW: float, aH: float) -> list[Flowable]: ...
    def split(self, aW: float, aH: float, /) -> list[Flowable]: ...
    def getKeepWithNext(self): ...
    def getSpaceAfter(self) -> float: ...
    def getSpaceBefore(self) -> float: ...
    def isIndexing(self) -> int: ...
    def identity(self, maxLen: int | None = None) -> str: ...

class XBox(Flowable):
    text: str
    def __init__(self, width: float, height: float, text: str = "A Box") -> None: ...
    def draw(self) -> None: ...

def splitLines(lines, maximum_length, split_characters, new_line_characters): ...
def splitLine(line_to_split, lines_splitted, maximum_length, split_characters, new_line_characters) -> None: ...

class Preformatted(Flowable):
    style: ParagraphStyle
    bulletText: str | None
    lines: list[str]
    def __init__(
        self,
        text: str,
        # NOTE: Technically has to be a ParagraphStyle, but that would
        #       conflict with stylesheet["Style"] usage
        style: PropertySet,
        bulletText: str | None = None,
        dedent: int = 0,
        maxLineLength: int | None = None,
        splitChars: str | None = None,
        newLineChars: str = "",
    ) -> None: ...
    def draw(self) -> None: ...

class Image(Flowable):
    filename: str
    # these are lazy, but __getattr__ ensures the image gets loaded
    # as soon as these attributes are accessed
    imageWidth: int
    imageHeight: int
    drawWidth: float
    drawHeight: float
    def __init__(
        self,
        # TODO: I think this might also accept a PIL.Image and other
        #       kinds of path represenations, should be kept in sync
        #       with reportlab.lib.utils.ImageReader, except for the
        #       potential PIL.Image shortcut
        filename: str | SupportsRead[bytes] | Incomplete,
        width: float | None = None,
        height: float | None = None,
        kind: str = "direct",
        mask: str = "auto",
        lazy: int = 1,
        hAlign: _HAlignment = "CENTER",
        useDPI: bool = False,
    ) -> None: ...
    def draw(self) -> None: ...

class NullDraw(Flowable):
    def draw(self) -> None: ...

class Spacer(NullDraw):
    # NOTE: This may actually be a bug, it seems likely that Spacer is meant
    #       to set spaceBefore in the isGlue case.
    spacebefore: float
    def __init__(self, width: float, height: float, isGlue: bool = False) -> None: ...

class UseUpSpace(NullDraw): ...

class PageBreak(UseUpSpace):
    locChanger: int
    nextTemplate: str | None
    def __init__(self, nextTemplate: str | None = None) -> None: ...

class SlowPageBreak(PageBreak): ...
class PageBreakIfNotEmpty(PageBreak): ...

class CondPageBreak(Spacer):
    locChanger: int
    def __init__(self, height: float) -> None: ...

class _ContainerSpace:
    def getSpaceBefore(self) -> float: ...
    def getSpaceAfter(self) -> float: ...

class KeepTogether(_ContainerSpace, Flowable):
    splitAtTop: bool
    # TODO: Consider using Sequence[Flowable] for covariance, even if reportlab
    #       only supports list/tuple
    def __init__(self, flowables: _FlowableSublist | None, maxHeight=None) -> None: ...

class KeepTogetherSplitAtTop(KeepTogether):
    splitAtTop: bool

class Macro(Flowable):
    command: str
    def __init__(self, command: str) -> None: ...
    def draw(self) -> None: ...

class CallerMacro(Flowable):
    def __init__(
        self,
        drawCallable: Callable[[CallerMacro, float, float], object] | None = None,
        wrapCallable: Callable[[CallerMacro, float, float], object] | None = None,
    ) -> None: ...
    def draw(self) -> None: ...

class ParagraphAndImage(Flowable):
    P: Paragraph
    I: Image
    xpad: float
    ypad: float
    def __init__(self, P: Paragraph, I: Image, xpad: float = 3, ypad: float = 3, side: str = "right") -> None: ...
    def draw(self) -> None: ...

class FailOnWrap(NullDraw):
    def wrap(self, aW: float, aH: float) -> NoReturn: ...

class FailOnDraw(Flowable):
    def draw(self) -> NoReturn: ...

class HRFlowable(Flowable):
    width: float | str  # type: ignore[assignment]
    lineWidth: float
    lineCap: str
    color: _Color
    dash: Incomplete | None
    def __init__(
        self,
        width: float | str = "80%",
        thickness: float = 1,
        lineCap: str = "round",
        color: _Color = ...,
        spaceBefore: float = 1,
        spaceAfter: float = 1,
        hAlign: _HAlignment = "CENTER",
        vAlign: _VAlignment = "BOTTOM",
        dash=None,
    ) -> None: ...
    def draw(self) -> None: ...

class _Container(_ContainerSpace):
    def drawOn(self, canv: Canvas, x: float, y: float) -> None: ...
    def copyContent(self, content: _FlowableSublist | None = None) -> None: ...

class PTOContainer(_Container, Flowable):
    def __init__(
        self, content: _FlowableSublist | None, trailer: _FlowableSublist | None = None, header: _FlowableSublist | None = None
    ) -> None: ...

class KeepInFrame(_Container, Flowable):
    name: str
    maxWidth: float
    maxHeight: float
    mode: Literal["error", "continue", "shrink", "truncate"]
    mergespace: Incomplete | None
    fakeWidth: bool | None
    def __init__(
        self,
        maxWidth: float,
        maxHeight: float,
        content: list[Flowable] = [],
        mergeSpace: Incomplete | None = 1,
        mode: Literal["error", "continue", "shrink", "truncate"] = "shrink",
        name: str = "",
        hAlign: str = "LEFT",
        vAlign: str = "BOTTOM",
        fakeWidth: bool | None = None,
    ) -> None: ...

class PlacedStory(Flowable):
    def __init__(
        self,
        x,
        y,
        maxWidth: float,
        maxHeight: float,
        content: list[Flowable] = [],
        mergeSpace: Incomplete | None = 1,
        mode: Literal["error", "continue", "shrink", "truncate"] = "shrink",
        name: str = "",
        anchor: str = "sw",
        fakeWidth: bool | None = None,
        hAlign: str = "LEFT",
        vAlign: str = "BOTTOM",
        showBoundary=None,
        origin="page",
    ) -> None: ...
    def wrap(self, _aW: Unused, _aH: Unused) -> tuple[Literal[0], Literal[0]]: ...
    def drawOn(self, canv: Canvas, lx: float, ly: float, _sW=0) -> None: ...

class _FindSplitterMixin: ...

class ImageAndFlowables(_Container, _FindSplitterMixin, Flowable):
    imageHref: str | None
    def __init__(
        self,
        I: Image,
        F: _FlowableSublist | None,
        imageLeftPadding: float = 0,
        imageRightPadding: float = 3,
        imageTopPadding: float = 0,
        imageBottomPadding: float = 3,
        imageSide: str = "right",
        imageHref: str | None = None,
    ) -> None: ...
    def deepcopy(self) -> Self: ...

class BalancedColumns(_FindSplitterMixin, NullDraw):
    name: str
    showBoundary: Incomplete | None
    endSlack: float
    def __init__(
        self,
        F: _FlowableSublist | None,
        nCols: int = 2,
        needed: float = 72,
        spaceBefore: float = 0,
        spaceAfter: float = 0,
        showBoundary=None,
        leftPadding: float | None = None,
        innerPadding: float | None = None,
        rightPadding: float | None = None,
        topPadding: float | None = None,
        bottomPadding: float | None = None,
        name: str = "",
        endSlack: float = 0.1,
        boxStrokeColor: Color | None = None,
        boxStrokeWidth: float = 0,
        boxFillColor: Color | None = None,
        boxMargin: tuple[int, int, int, int] | tuple[int, int, int] | tuple[int, int] | tuple[int] | None = None,
        vLinesStrokeColor: Color | None = None,
        vLinesStrokeWidth: float | None = None,
    ) -> None: ...

class AnchorFlowable(Spacer):
    def __init__(self, name: str) -> None: ...

class FrameBG(AnchorFlowable):
    start: bool
    left: float
    right: float
    color: Color
    strokeWidth: float
    strokeColor: Color
    strokeDashArray: list[float] | tuple[float, ...] | None
    def __init__(
        self,
        color: Color | None = None,
        left: float | str = 0,
        right: float | str = 0,
        start: bool = True,
        strokeWidth: float | None = None,
        strokeColor: Color | None = None,
        strokeDashArray: list[float] | tuple[float, ...] | None = None,
    ) -> None: ...

class FrameSplitter(NullDraw):
    nextTemplate: str
    nextFrames: list[str]
    gap: float
    required: float
    adjustHeight: bool
    def __init__(
        self,
        nextTemplate: str,
        nextFrames: list[str] | None = [],
        gap: float = 10,
        required: float = 72,
        adjustHeight: bool = True,
    ) -> None: ...

class BulletDrawer:
    value: str
    def __init__(
        self,
        value: str = "0",
        bulletAlign: str = "left",
        bulletType: str = "1",
        bulletColor: str = "black",
        bulletFontName: str = "Helvetica",
        bulletFontSize: int = 12,
        bulletOffsetY: int = 0,
        bulletDedent: int = 0,
        bulletDir: str = "ltr",
        bulletFormat=None,
    ) -> None: ...
    def drawOn(self, indenter: DDIndenter, canv: Canvas, x: float, y: float) -> None: ...

class DDIndenter(Flowable):
    def __init__(self, flowable: Flowable, leftIndent: float = 0, rightIndent: float = 0) -> None: ...

class LIIndenter(DDIndenter):
    def __init__(
        self,
        flowable: Flowable,
        leftIndent: float = 0,
        rightIndent: float = 0,
        bullet=None,
        spaceBefore: float | None = None,
        spaceAfter: float | None = None,
    ) -> None: ...

class ListItem:
    # NOTE: style has to be a ListStyle, but this will be annoying with sheet["ul"]
    # TODO: Use Unpack for kwds with the ListStyle properties + value/spaceBefore/spaceAfter
    def __init__(self, flowables: _FlowableSublist, style: PropertySet | None = None, **kwds) -> None: ...

class ListFlowable(_Container, Flowable):
    style: ListStyle
    # NOTE: style has to be a ListStyle, but this will be annoying with sheet["ul"]
    # TODO: Use Unpack for kwds with the ListStyle properties + spaceBefore/spaceAfter
    def __init__(self, flowables: Iterable[_NestedFlowable], start=None, style: PropertySet | None = None, **kwds) -> None: ...

class TopPadder(Flowable):
    # NOTE: TopPadder is mostly a transparent wrapper, we may consider trying
    #       something using __new__ in the future
    def __init__(self, f: Flowable) -> None: ...
    def __setattr__(self, a: str, v: Any) -> None: ...
    def __getattr__(self, a: str) -> Any: ...
    def __delattr__(self, a: str) -> None: ...

class DocAssign(NullDraw):
    args: tuple[Any, ...]
    def __init__(self, var: str, expr: object, life: str = "forever") -> None: ...
    def funcWrap(self, aW: float, aH: float) -> None: ...
    def func(self) -> None: ...

class DocExec(DocAssign):
    def __init__(self, stmt: str, lifetime: str = "forever") -> None: ...

class DocPara(DocAssign):
    expr: object
    format: str | None
    style: PropertySet | None
    klass: Incomplete
    escape: bool
    def __init__(
        self,
        expr: object,
        format: str | None = None,
        style: PropertySet | None = None,
        klass: _StyledFlowableFactory | None = None,
        escape: bool = True,
    ) -> None: ...
    def funcWrap(self, aW: float, aH: float) -> Any: ...
    def func(self) -> Any: ...
    def add_content(self, *args: Flowable) -> None: ...
    def get_value(self, aW: float, aH: float) -> str: ...

class DocAssert(DocPara):
    def __init__(self, cond: object, format: str | None = None) -> None: ...

class DocIf(DocPara):
    blocks: tuple[_FlowableSublist, _FlowableSublist]
    def __init__(self, cond: object, thenBlock: _FlowableSublist, elseBlock: _FlowableSublist = []) -> None: ...
    def checkBlock(self, block: _FlowableSublist) -> list[Flowable] | tuple[Flowable, ...]: ...

class DocWhile(DocIf):
    block: _FlowableSublist
    def __init__(self, cond: object, whileBlock: _FlowableSublist) -> None: ...

class SetTopFlowables(NullDraw):
    def __init__(self, F: list[Flowable], show: bool = False) -> None: ...

class SetPageTopFlowables(NullDraw):
    def __init__(self, F: list[Flowable], show: bool = False) -> None: ...



from collections import OrderedDict
from collections.abc import Sequence
from enum import Enum
from typing import Any, ClassVar

from django.db import models
from django.db.models import Model, QuerySet
from django.forms import Form
from django.http import HttpRequest, QueryDict

from .filters import Filter

def remote_queryset(field: models.Field[Any, Any]) -> QuerySet[Any]: ...  # Field type params vary by model definition

class UnknownFieldBehavior(Enum):
    RAISE = "raise"
    WARN = "warn"
    IGNORE = "ignore"

class FilterSetOptions:
    model: type[Model] | None
    fields: Sequence[str] | dict[str, Sequence[str]] | str | None
    exclude: Sequence[str] | None
    filter_overrides: dict[type[models.Field[Any, Any]], dict[str, Any]]  # Field override mapping
    form: type[Form]
    unknown_field_behavior: UnknownFieldBehavior
    def __init__(self, options: Any | None = None) -> None: ...  # Meta options can be various configuration types

class FilterSetMetaclass(type):
    # Class attrs vary by definition
    def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> FilterSetMetaclass: ...

    # Class attrs vary by definition
    @classmethod
    def get_declared_filters(cls, bases: tuple[type, ...], attrs: dict[str, Any]) -> OrderedDict[str, Filter]: ...

# Django field types vary widely - Any allows mapping all field types to their filters
FILTER_FOR_DBFIELD_DEFAULTS: dict[type[models.Field[Any, Any]], dict[str, Any]]

class BaseFilterSet:
    FILTER_DEFAULTS: ClassVar[dict[type[models.Field[Any, Any]], dict[str, Any]]] = ...  # Field type mapping
    is_bound: bool
    base_filters: OrderedDict[str, Filter]
    declared_filters: OrderedDict[str, Filter]
    data: QueryDict | dict[str, Any] | None  # Filter input data values vary
    queryset: QuerySet[Any] | None  # Base queryset for any model type
    request: HttpRequest | None
    form_prefix: str | None
    filters: OrderedDict[str, Filter]
    def __init__(
        self,
        data: QueryDict | dict[str, Any] | None = None,  # Filter data values vary
        queryset: QuerySet[Any] | None = None,  # Base queryset for any model
        *,
        request: HttpRequest | None = None,
        prefix: str | None = None,
    ) -> None: ...
    def is_valid(self) -> bool: ...
    @property
    def errors(self) -> dict[str, list[str]]: ...
    def filter_queryset(self, queryset: QuerySet[Any]) -> QuerySet[Any]: ...  # Works with any model type
    @property
    def qs(self) -> QuerySet[Any]: ...  # Filtered queryset of any model
    def get_form_class(self) -> type[Form]: ...
    @property
    def form(self) -> Form: ...
    @classmethod
    def get_fields(cls) -> dict[str, models.Field[Any, Any]]: ...  # Model fields have varying type params
    @classmethod
    def get_filter_name(cls, field_name: str, lookup_expr: str) -> str: ...
    @classmethod
    def get_filters(cls) -> OrderedDict[str, Filter]: ...
    @classmethod
    def handle_unrecognized_field(cls, field_name: str, message: str) -> None: ...
    @classmethod
    def filter_for_field(
        cls, field: models.Field[Any, Any], field_name: str, lookup_expr: str | None = None
    ) -> Filter: ...  # Accepts any Django field type
    @classmethod
    def filter_for_lookup(cls, field: models.Field[Any, Any], lookup_type: str) -> type[Filter]: ...  # Field type varies by model

class FilterSet(BaseFilterSet, metaclass=FilterSetMetaclass): ...

def filterset_factory(
    model: type[Model], filterset: FilterSetMetaclass = ..., fields: Sequence[str] | dict[str, Sequence[str]] | str | None = None
) -> type[FilterSet]: ...



from _typeshed import Incomplete
from collections.abc import Hashable, Iterable
from typing import TypeVar

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["union", "compose", "disjoint_union", "intersection", "difference", "symmetric_difference", "full_join"]

@_dispatchable
def disjoint_union(G: Graph[_Node], H: Graph[_Node]): ...
@_dispatchable
def intersection(G: Graph[_Node], H: Graph[_Node]): ...
@_dispatchable
def difference(G: Graph[_Node], H: Graph[_Node]): ...
@_dispatchable
def symmetric_difference(G: Graph[_Node], H: Graph[_Node]): ...

_X_co = TypeVar("_X_co", bound=Hashable, covariant=True)
_Y_co = TypeVar("_Y_co", bound=Hashable, covariant=True)

@_dispatchable
def compose(G: Graph[_X_co], H: Graph[_Y_co]) -> DiGraph[_X_co | _Y_co]: ...
@_dispatchable
def full_join(G: Graph[_Node], H, rename=(None, None)): ...
@_dispatchable
def union(G: Graph[_X_co], H: Graph[_Y_co], rename: Iterable[Incomplete] | None = ()) -> DiGraph[_X_co | _Y_co]: ...



from _typeshed import SupportsRead
from typing import Final
from xml.dom.expatbuilder import ExpatBuilder as _ExpatBuilder, Namespaces as _Namespaces
from xml.dom.minidom import Document
from xml.dom.xmlbuilder import Options
from xml.parsers.expat import XMLParserType

__origin__: Final = "xml.dom.expatbuilder"

class DefusedExpatBuilder(_ExpatBuilder):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(
        self, options: Options | None = None, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True
    ) -> None: ...
    def defused_start_doctype_decl(self, name: str, sysid: str | None, pubid: str | None, has_internal_subset: bool) -> None: ...
    def defused_entity_decl(
        self,
        name: str,
        is_parameter_entity: bool,
        value: str | None,
        base: str | None,
        sysid: str,
        pubid: str | None,
        notation_name: str | None,
    ) -> None: ...
    def defused_unparsed_entity_decl(
        self, name: str, base: str | None, sysid: str, pubid: str | None, notation_name: str
    ) -> None: ...
    def defused_external_entity_ref_handler(
        self, context: str, base: str | None, sysid: str | None, pubid: str | None
    ) -> None: ...
    def install(self, parser: XMLParserType) -> None: ...

class DefusedExpatBuilderNS(_Namespaces, DefusedExpatBuilder):
    def install(self, parser: XMLParserType) -> None: ...
    def reset(self) -> None: ...

def parse(
    file: str | SupportsRead[bytes | str],
    namespaces: bool = True,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> Document: ...
def parseString(
    string: str, namespaces: bool = True, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True
) -> Document: ...



"""Bytecode caching for Jac modules.

This module provides disk-based caching for compiled Jac bytecode,
similar to Python's __pycache__ mechanism. Cache files are stored
in a global user cache directory following platform conventions:
- Linux:   ~/.cache/jac/bytecode/
- macOS:   ~/Library/Caches/jac/bytecode/
- Windows: %LOCALAPPDATA%/jac/cache/bytecode/
"""

from __future__ import annotations

import hashlib
import marshal
import os
import sys
import types
from dataclasses import dataclass
from pathlib import Path
from typing import TYPE_CHECKING, Final

if TYPE_CHECKING:
    from jaclang.project.config import JacConfig


def get_global_cache_dir() -> Path:
    """Get the platform-appropriate global cache directory for Jac bytecode.

    Returns:
        Path to the global cache directory:
        - Linux:   ~/.cache/jac/bytecode/ (respects XDG_CACHE_HOME)
        - macOS:   ~/Library/Caches/jac/bytecode/
        - Windows: %LOCALAPPDATA%/jac/cache/bytecode/
    """
    if sys.platform == "win32":
        # Windows: Use LOCALAPPDATA
        base = Path(os.environ.get("LOCALAPPDATA", Path.home() / "AppData" / "Local"))
        return base / "jac" / "cache" / "bytecode"
    elif sys.platform == "darwin":
        # macOS: Use ~/Library/Caches
        return Path.home() / "Library" / "Caches" / "jac" / "bytecode"
    else:
        # Linux/Unix: Follow XDG Base Directory Specification
        xdg_cache = os.environ.get("XDG_CACHE_HOME")
        base = Path(xdg_cache) if xdg_cache else Path.home() / ".cache"
        return base / "jac" / "bytecode"


def discover_annex_files(source_path: str, suffix: str = ".impl.jac") -> list[str]:
    """Discover annex files (.impl.jac, .test.jac) for a source .jac or .cl.jac file.

    Searches: same directory, module-specific folder (foo.impl/), shared folder (impl/, test/).
    """
    src = Path(source_path).resolve()
    # Skip non-.jac files and files that are the same annex type as requested
    if not src.name.endswith(".jac") or src.name.endswith(suffix):
        return []

    # Handle .cl.jac files: extract base name (e.g., foo.cl.jac -> foo)
    base = src.stem
    if base.endswith(".cl"):
        base = base[:-3]
    mod_folder = src.parent / (base + suffix[:-4])  # foo.impl/, foo.test/
    shared_folder = suffix[1:-4]  # Extract "impl" or "test"
    dirs = [src.parent, mod_folder, src.parent / shared_folder]
    return [
        str(f)
        for d in dirs
        if d.is_dir()
        for f in d.iterdir()
        if f.is_file()
        and f.name.endswith(suffix)
        and (d == mod_folder or f.name.startswith(f"{base}."))
    ]


def discover_base_file(annex_path: str) -> str | None:
    """Discover the base .jac or .cl.jac file for an annex file (.impl.jac, .test.jac).

    Searches: same directory, module-specific folder (foo.impl/), shared folder (impl/).
    For multi-part names like "foo.bar.impl.jac", only the first component is used.
    """
    src = Path(annex_path).resolve()
    annex_types = {".impl.jac": ".impl", ".test.jac": ".test"}

    # Find matching annex type and extract base name
    for suffix, folder_suffix in annex_types.items():
        if src.name.endswith(suffix):
            base_name = src.name[: -len(suffix)].split(".")[0]
            parent = src.parent.name
            # Build search candidates: (directory, base_name_to_search)
            candidates = [(src.parent, base_name)]
            if parent.endswith(folder_suffix):  # Module-specific folder
                candidates.append((src.parent.parent, parent[: -len(folder_suffix)]))
            if parent in {"impl", "test"}:  # Shared folder
                candidates.append((src.parent.parent, base_name))
            # Search for base file (both .jac and .cl.jac can have annexes)
            for directory, name in candidates:
                for ext in (".jac", ".cl.jac"):
                    path = directory / f"{name}{ext}"
                    if path.is_file() and path != src:
                        return str(path)
            break
    return None


@dataclass(frozen=True, slots=True)
class CacheKey:
    """Immutable key identifying a cached bytecode entry.

    Attributes:
        source_path: Absolute path to the source .jac file.
        minimal: Whether minimal compilation mode was used.
        python_version: Python version tuple (major, minor).
    """

    source_path: str
    minimal: bool
    python_version: tuple[int, int]

    @classmethod
    def for_source(cls, source_path: str, minimal: bool = False) -> CacheKey:
        """Create a cache key for the current Python version."""
        return cls(
            source_path=source_path,
            minimal=minimal,
            python_version=(sys.version_info.major, sys.version_info.minor),
        )


class BytecodeCache:
    """Abstract interface for bytecode caching."""

    def get(self, _key: CacheKey) -> types.CodeType | None:
        """Retrieve cached bytecode if valid."""
        raise NotImplementedError

    def put(self, _key: CacheKey, _bytecode: bytes) -> None:
        """Store bytecode in the cache."""
        raise NotImplementedError


class DiskBytecodeCache(BytecodeCache):
    """Disk-based bytecode cache using a global user cache directory.

    Cache files are stored in a platform-appropriate global location:
    - Linux:   ~/.cache/jac/bytecode/
    - macOS:   ~/Library/Caches/jac/bytecode/
    - Windows: %LOCALAPPDATA%/jac/cache/bytecode/

    Filenames include a path hash, Python version, and compilation mode
    to ensure uniqueness across different projects and avoid conflicts.

    Example:
        source:  /project/src/main.jac
        cache:   ~/.cache/jac/bytecode/main.a1b2c3d4.cpython-312.jbc
                 ~/.cache/jac/bytecode/main.a1b2c3d4.cpython-312.minimal.jbc
    """

    EXTENSION: Final[str] = ".jbc"
    MINIMAL_SUFFIX: Final[str] = ".minimal"

    def __init__(self, config: JacConfig | None = None) -> None:
        """Initialize the cache with optional config."""
        self._config = config
        self._cache_dir: Path | None = None

    def _get_cache_dir(self) -> Path:
        """Get the global cache directory."""
        if self._cache_dir is not None:
            return self._cache_dir

        self._cache_dir = get_global_cache_dir()
        return self._cache_dir

    def _get_cache_path(self, key: CacheKey) -> Path:
        """Generate the cache file path for a given key.

        Uses a hash of the full source path to ensure uniqueness when
        files with the same name exist in different directories.
        """
        source = Path(key.source_path).resolve()
        cache_dir = self._get_cache_dir()

        # Create a short hash of the full path for uniqueness
        path_hash = hashlib.sha256(str(source).encode()).hexdigest()[:8]

        major, minor = key.python_version
        py_version = f"cpython-{major}{minor}"
        suffix = (
            f"{self.MINIMAL_SUFFIX}{self.EXTENSION}" if key.minimal else self.EXTENSION
        )
        cache_name = f"{source.stem}.{path_hash}.{py_version}{suffix}"

        return cache_dir / cache_name

    def _is_valid(self, key: CacheKey, cache_path: Path) -> bool:
        """Check if cached bytecode is still valid.

        The cache is valid if:
        - The cache file exists
        - The cache is newer than the source file
        - The cache is newer than all impl files associated with the source
        """
        if not cache_path.exists():
            return False

        try:
            cache_mtime = os.path.getmtime(cache_path)

            # Check source file modification time
            source_mtime = os.path.getmtime(key.source_path)
            if cache_mtime <= source_mtime:
                return False

            # Check all impl files - cache must be newer than all of them
            for impl_path in discover_annex_files(key.source_path):
                try:
                    impl_mtime = os.path.getmtime(impl_path)
                    if cache_mtime <= impl_mtime:
                        return False
                except OSError:
                    # If we can't stat an impl file, invalidate cache to be safe
                    return False

            return True
        except OSError:
            return False

    def get(self, key: CacheKey) -> types.CodeType | None:
        """Retrieve cached bytecode if valid."""
        cache_path = self._get_cache_path(key)

        if not self._is_valid(key, cache_path):
            return None

        try:
            bytecode = cache_path.read_bytes()
            return marshal.loads(bytecode)
        except (OSError, ValueError, EOFError):
            return None

    def put(self, key: CacheKey, bytecode: bytes) -> None:
        """Store bytecode in the cache."""
        cache_path = self._get_cache_path(key)

        try:
            cache_path.parent.mkdir(parents=True, exist_ok=True)
            cache_path.write_bytes(bytecode)
        except OSError:
            pass  # Silently ignore write failures


# Default cache instance (singleton)
_default_cache: BytecodeCache | None = None


def get_bytecode_cache() -> BytecodeCache:
    """Get the default bytecode cache instance."""
    global _default_cache
    if _default_cache is None:
        _default_cache = DiskBytecodeCache()
    return _default_cache



# The types here are all undocumented, but all feature columns are return types of the
# public functions in tf.feature_column. As they are undocumented internals while some
# common methods are included, they are incomplete and do not have getattr Incomplete fallback.

from _typeshed import Incomplete
from abc import ABC, ABCMeta, abstractmethod
from collections.abc import Callable, Sequence
from typing import Literal
from typing_extensions import Self, TypeAlias

import tensorflow as tf
from tensorflow._aliases import ShapeLike

_Combiners: TypeAlias = Literal["mean", "sqrtn", "sum"]
_ExampleSpec: TypeAlias = dict[str, tf.io.FixedLenFeature | tf.io.VarLenFeature]

class _FeatureColumn(ABC):
    @property
    @abstractmethod
    def name(self) -> str: ...
    @property
    @abstractmethod
    def parse_example_spec(self) -> _ExampleSpec: ...
    def __lt__(self, other: _FeatureColumn) -> bool: ...
    def __gt__(self, other: _FeatureColumn) -> bool: ...
    @property
    @abstractmethod
    def parents(self) -> list[_FeatureColumn | str]: ...

class DenseColumn(_FeatureColumn, metaclass=ABCMeta): ...
class SequenceDenseColumn(_FeatureColumn, metaclass=ABCMeta): ...

# These classes are mostly subclasses of collections.namedtuple but we can't use
# typing.NamedTuple because they use multiple inheritance with other non namedtuple classes.
# _cls instead of cls is because collections.namedtuple uses _cls for __new__.
class NumericColumn(DenseColumn):
    key: str
    shape: ShapeLike
    default_value: float
    dtype: tf.DType
    normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None

    def __new__(
        _cls,
        key: str,
        shape: ShapeLike,
        default_value: float,
        dtype: tf.DType,
        normalizer_fn: Callable[[tf.Tensor], tf.Tensor] | None,
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class CategoricalColumn(_FeatureColumn):
    @property
    @abstractmethod
    def num_buckets(self) -> int: ...

class BucketizedColumn(DenseColumn, CategoricalColumn):
    source_column: NumericColumn
    boundaries: list[float] | tuple[float, ...]

    def __new__(_cls, source_column: NumericColumn, boundaries: list[float] | tuple[float, ...]) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class EmbeddingColumn(DenseColumn, SequenceDenseColumn):
    categorical_column: CategoricalColumn
    dimension: int
    combiner: _Combiners
    initializer: Callable[[ShapeLike], tf.Tensor] | None
    ckpt_to_load_from: str | None
    tensor_name_in_ckpt: str | None
    max_norm: float | None
    trainable: bool
    use_safe_embedding_lookup: bool

    # This one subclasses collections.namedtuple and overrides __new__.
    def __new__(
        cls,
        categorical_column: CategoricalColumn,
        dimension: int,
        combiner: _Combiners,
        initializer: Callable[[ShapeLike], tf.Tensor] | None,
        ckpt_to_load_from: str | None,
        tensor_name_in_ckpt: str | None,
        max_norm: float | None,
        trainable: bool,
        use_safe_embedding_lookup: bool = True,
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class SharedEmbeddingColumnCreator:
    def __init__(
        self,
        dimension: int,
        initializer: Callable[[ShapeLike], tf.Tensor] | None,
        ckpt_to_load_from: str | None,
        tensor_name_in_ckpt: str | None,
        num_buckets: int,
        trainable: bool,
        name: str = "shared_embedding_column_creator",
        use_safe_embedding_lookup: bool = True,
    ) -> None: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class SharedEmbeddingColumn(DenseColumn, SequenceDenseColumn):
    categorical_column: CategoricalColumn
    shared_embedding_column_creator: SharedEmbeddingColumnCreator
    combiner: _Combiners
    max_norm: float | None
    use_safe_embedding_lookup: bool

    def __new__(
        cls,
        categorical_column: CategoricalColumn,
        shared_embedding_column_creator: SharedEmbeddingColumnCreator,
        combiner: _Combiners,
        max_norm: float | None,
        use_safe_embedding_lookup: bool = True,
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class CrossedColumn(CategoricalColumn):
    keys: tuple[str, ...]
    hash_bucket_size: int
    hash_key: int | None

    def __new__(_cls, keys: tuple[str, ...], hash_bucket_size: int, hash_key: int | None) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class IdentityCategoricalColumn(CategoricalColumn):
    key: str
    number_buckets: int
    default_value: int | None

    def __new__(_cls, key: str, number_buckets: int, default_value: int | None) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class HashedCategoricalColumn(CategoricalColumn):
    key: str
    hash_bucket_size: int
    dtype: tf.DType

    def __new__(_cls, key: str, hash_bucket_size: int, dtype: tf.DType) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class VocabularyFileCategoricalColumn(CategoricalColumn):
    key: str
    vocabulary_file: str
    vocabulary_size: int | None
    num_oov_buckets: int
    dtype: tf.DType
    default_value: str | int | None
    file_format: str | None

    def __new__(
        cls,
        key: str,
        vocabulary_file: str,
        vocabulary_size: int | None,
        num_oov_buckets: int,
        dtype: tf.DType,
        default_value: str | int | None,
        file_format: str | None = None,
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class VocabularyListCategoricalColumn(CategoricalColumn):
    key: str
    vocabulary_list: Sequence[str] | Sequence[int]
    dtype: tf.DType
    default_value: str | int | None
    num_oov_buckets: int

    def __new__(
        _cls, key: str, vocabulary_list: Sequence[str], dtype: tf.DType, default_value: str | int | None, num_oov_buckets: int
    ) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class WeightedCategoricalColumn(CategoricalColumn):
    categorical_column: CategoricalColumn
    weight_feature_key: str
    dtype: tf.DType

    def __new__(_cls, categorical_column: CategoricalColumn, weight_feature_key: str, dtype: tf.DType) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class IndicatorColumn(DenseColumn, SequenceDenseColumn):
    categorical_column: CategoricalColumn

    def __new__(_cls, categorical_column: CategoricalColumn) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...

class SequenceCategoricalColumn(CategoricalColumn):
    categorical_column: CategoricalColumn

    def __new__(_cls, categorical_column: CategoricalColumn) -> Self: ...
    @property
    def name(self) -> str: ...
    @property
    def num_buckets(self) -> int: ...
    @property
    def parse_example_spec(self) -> _ExampleSpec: ...
    @property
    def parents(self) -> list[_FeatureColumn | str]: ...



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Seal(SystemBackendMixin):
    def is_sealed(self): ...
    def read_seal_status(self): ...
    def seal(self): ...
    def submit_unseal_key(self, key=None, reset: bool = False, migrate: bool = False): ...
    def submit_unseal_keys(self, keys, migrate: bool = False): ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import ClassVar, final

from google._upb._message import Descriptor, FileDescriptor, MessageMeta
from google.protobuf import message
from google.protobuf.internal import containers

DESCRIPTOR: FileDescriptor

@final
class ServerReflectionRequest(message.Message, metaclass=MessageMeta):
    HOST_FIELD_NUMBER: ClassVar[int]
    FILE_BY_FILENAME_FIELD_NUMBER: ClassVar[int]
    FILE_CONTAINING_SYMBOL_FIELD_NUMBER: ClassVar[int]
    FILE_CONTAINING_EXTENSION_FIELD_NUMBER: ClassVar[int]
    ALL_EXTENSION_NUMBERS_OF_TYPE_FIELD_NUMBER: ClassVar[int]
    LIST_SERVICES_FIELD_NUMBER: ClassVar[int]
    host: str
    file_by_filename: str
    file_containing_symbol: str
    file_containing_extension: ExtensionRequest
    all_extension_numbers_of_type: str
    list_services: str
    def __init__(
        self,
        host: str | None = ...,
        file_by_filename: str | None = ...,
        file_containing_symbol: str | None = ...,
        file_containing_extension: ExtensionRequest | Mapping[Incomplete, Incomplete] | None = ...,
        all_extension_numbers_of_type: str | None = ...,
        list_services: str | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ExtensionRequest(message.Message, metaclass=MessageMeta):
    CONTAINING_TYPE_FIELD_NUMBER: ClassVar[int]
    EXTENSION_NUMBER_FIELD_NUMBER: ClassVar[int]
    containing_type: str
    extension_number: int
    def __init__(self, containing_type: str | None = ..., extension_number: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ServerReflectionResponse(message.Message, metaclass=MessageMeta):
    VALID_HOST_FIELD_NUMBER: ClassVar[int]
    ORIGINAL_REQUEST_FIELD_NUMBER: ClassVar[int]
    FILE_DESCRIPTOR_RESPONSE_FIELD_NUMBER: ClassVar[int]
    ALL_EXTENSION_NUMBERS_RESPONSE_FIELD_NUMBER: ClassVar[int]
    LIST_SERVICES_RESPONSE_FIELD_NUMBER: ClassVar[int]
    ERROR_RESPONSE_FIELD_NUMBER: ClassVar[int]
    valid_host: str
    original_request: ServerReflectionRequest
    file_descriptor_response: FileDescriptorResponse
    all_extension_numbers_response: ExtensionNumberResponse
    list_services_response: ListServiceResponse
    error_response: ErrorResponse
    def __init__(
        self,
        valid_host: str | None = ...,
        original_request: ServerReflectionRequest | Mapping[Incomplete, Incomplete] | None = ...,
        file_descriptor_response: FileDescriptorResponse | Mapping[Incomplete, Incomplete] | None = ...,
        all_extension_numbers_response: ExtensionNumberResponse | Mapping[Incomplete, Incomplete] | None = ...,
        list_services_response: ListServiceResponse | Mapping[Incomplete, Incomplete] | None = ...,
        error_response: ErrorResponse | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class FileDescriptorResponse(message.Message, metaclass=MessageMeta):
    FILE_DESCRIPTOR_PROTO_FIELD_NUMBER: ClassVar[int]
    file_descriptor_proto: containers.RepeatedScalarFieldContainer[bytes]
    def __init__(self, file_descriptor_proto: Iterable[bytes] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ExtensionNumberResponse(message.Message, metaclass=MessageMeta):
    BASE_TYPE_NAME_FIELD_NUMBER: ClassVar[int]
    EXTENSION_NUMBER_FIELD_NUMBER: ClassVar[int]
    base_type_name: str
    extension_number: containers.RepeatedScalarFieldContainer[int]
    def __init__(self, base_type_name: str | None = ..., extension_number: Iterable[int] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ListServiceResponse(message.Message, metaclass=MessageMeta):
    SERVICE_FIELD_NUMBER: ClassVar[int]
    service: containers.RepeatedCompositeFieldContainer[ServiceResponse]
    def __init__(self, service: Iterable[ServiceResponse | Mapping[Incomplete, Incomplete]] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ServiceResponse(message.Message, metaclass=MessageMeta):
    NAME_FIELD_NUMBER: ClassVar[int]
    name: str
    def __init__(self, name: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ErrorResponse(message.Message, metaclass=MessageMeta):
    ERROR_CODE_FIELD_NUMBER: ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: ClassVar[int]
    error_code: int
    error_message: str
    def __init__(self, error_code: int | None = ..., error_message: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor



"""Line-cache functionality."""

import linecache
from typing import List


def generate_unique_filename(cls: type, func_name: str, lines: List[str] = []) -> str:
    """
    Create a "filename" suitable for a function being generated.

    If *lines* are provided, insert them in the first free spot or stop
    if a duplicate is found.
    """
    extra = ""
    count = 1

    while True:
        unique_filename = "<cattrs generated {} {}.{}{}>".format(
            func_name, cls.__module__, getattr(cls, "__qualname__", cls.__name__), extra
        )
        if not lines:
            return unique_filename
        cache_line = (len("\n".join(lines)), None, lines, unique_filename)
        if linecache.cache.setdefault(unique_filename, cache_line) == cache_line:
            return unique_filename

        # Looks like this spot is taken. Try again.
        count += 1
        extra = f"-{count}"



from collections.abc import Callable
from dataclasses import dataclass

from seaborn._core.typing import Vector
from seaborn._stats.base import Stat

@dataclass
class Agg(Stat):
    func: str | Callable[[Vector], float] = "mean"

@dataclass
class Est(Stat):
    func: str | Callable[[Vector], float] = "mean"
    errorbar: str | tuple[str, float] = ("ci", 95)
    n_boot: int = 1000
    seed: int | None = None

@dataclass
class Rolling(Stat): ...



from typing import Literal

from gdb import Progspace

class MissingFileHandler:
    @property
    def name(self) -> str: ...
    enabled: bool
    def __init__(self, name: str) -> None: ...

def register_handler(
    handler_type: Literal["debug", "objfile"], locus: Progspace | None, handler: MissingFileHandler, replace: bool = False
) -> None: ...



import re
from _typeshed import Incomplete
from typing import Any, Final, overload

PARSER_HARDCODED_TOKENS: Final[list[str]]
PARSER_KNOWN_TOKENS: Final[list[str]]
ALWAYS_KEEP_TOKENS: Final[list[str]]
KNOWN_WORD_TOKENS: Final[list[str]]
PARENTHESES_PATTERN: Final[re.Pattern[str]]
NUMERAL_PATTERN: Final[re.Pattern[str]]
KEEP_TOKEN_PATTERN: Final[re.Pattern[str]]

class UnknownTokenError(Exception): ...

class Dictionary:
    info: Any
    def __init__(self, locale_info: dict[str, Incomplete], settings=None) -> None: ...
    def __contains__(self, key): ...
    def __getitem__(self, key): ...
    def __iter__(self) -> Any: ...
    def are_tokens_valid(self, tokens: list[str]) -> bool: ...
    @overload
    def split(self, string: None, keep_formatting: bool = False) -> None: ...
    @overload
    def split(self, string: str, keep_formatting: bool = False) -> list[str]: ...

class NormalizedDictionary(Dictionary):
    def __init__(self, locale_info: dict[str, Incomplete], settings=None) -> None: ...



from _typeshed import Incomplete

class Flake8Exception(Exception): ...
class EarlyQuit(Flake8Exception): ...
class ExecutionError(Flake8Exception): ...

class FailedToLoadPlugin(Flake8Exception):
    FORMAT: str
    plugin_name: Incomplete
    original_exception: Incomplete
    def __init__(self, plugin_name: str, exception: Exception) -> None: ...

class PluginRequestedUnknownParameters(Flake8Exception):
    FORMAT: str
    plugin_name: Incomplete
    original_exception: Incomplete
    def __init__(self, plugin_name: str, exception: Exception) -> None: ...

class PluginExecutionFailed(Flake8Exception):
    FORMAT: str
    filename: Incomplete
    plugin_name: Incomplete
    original_exception: Incomplete
    def __init__(self, filename: str, plugin_name: str, exception: Exception) -> None: ...



from .database import Database
from .delegated import Delegated
from .enterprise import Enterprise
from .get_token import GetToken
from .passwordless import Passwordless
from .revoke_token import RevokeToken
from .social import Social
from .users import Users

__all__ = ("Database", "Delegated", "Enterprise", "GetToken", "Passwordless", "RevokeToken", "Social", "Users")



from _typeshed import Incomplete
from typing import Any, Literal, overload
from xml.etree.ElementTree import Element

from ._inputstream import _InputStream
from ._tokenizer import HTMLTokenizer
from .treebuilders.base import TreeBuilder

@overload
def parse(
    doc: _InputStream, treebuilder: Literal["etree"] = "etree", namespaceHTMLElements: bool = True, **kwargs
) -> Element: ...
@overload
def parse(doc: _InputStream, treebuilder: str, namespaceHTMLElements: bool = True, **kwargs): ...
def parseFragment(
    doc: _InputStream, container: str = "div", treebuilder: str = "etree", namespaceHTMLElements: bool = True, **kwargs
): ...
def method_decorator_metaclass(function): ...

class HTMLParser:
    strict: bool
    tree: Incomplete
    errors: list[Incomplete]
    phases: Incomplete
    def __init__(
        self,
        tree: str | type[TreeBuilder] | None = None,
        strict: bool = False,
        namespaceHTMLElements: bool = True,
        debug: bool = False,
    ) -> None: ...
    firstStartTag: bool
    log: Incomplete
    compatMode: str
    container: str
    innerHTML: Incomplete
    phase: Incomplete
    lastPhase: Incomplete
    beforeRCDataPhase: Incomplete
    framesetOK: bool
    tokenizer: HTMLTokenizer
    def reset(self) -> None: ...
    @property
    def documentEncoding(self) -> str | None: ...
    def isHTMLIntegrationPoint(self, element: Element) -> bool: ...
    def isMathMLTextIntegrationPoint(self, element: Element) -> bool: ...
    def mainLoop(self) -> None: ...
    def parse(self, stream: _InputStream, scripting: bool = ..., **kwargs): ...
    def parseFragment(self, stream: _InputStream, *args, **kwargs): ...
    def parseError(self, errorcode: str = "XXX-undefined-error", datavars=None) -> None: ...
    def adjustMathMLAttributes(self, token: dict[str, Any]) -> None: ...
    def adjustSVGAttributes(self, token: dict[str, Any]) -> None: ...
    def adjustForeignAttributes(self, token: dict[str, Any]) -> None: ...
    def reparseTokenNormal(self, token: dict[str, Any]) -> None: ...
    def resetInsertionMode(self) -> None: ...
    originalPhase: Incomplete
    def parseRCDataRawtext(self, token, contentType: Literal["RAWTEXT", "RCDATA"]) -> None: ...

def getPhases(debug: bool | None) -> dict[str, type]: ...
def adjust_attributes(token: dict[str, Any], replacements: dict[str, Any]) -> None: ...
def impliedTagToken(
    name: str, type: str = "EndTag", attributes: dict[str, Any] | None = None, selfClosing: bool = False
) -> dict[str, Any]: ...

class ParseError(Exception): ...



from _typeshed import Incomplete

from antlr4 import DFA as DFA
from antlr4.atn.ATNConfigSet import ATNConfigSet as ATNConfigSet
from antlr4.error.ErrorListener import ErrorListener as ErrorListener

class DiagnosticErrorListener(ErrorListener):
    exactOnly: Incomplete
    def __init__(self, exactOnly: bool = True) -> None: ...
    def reportAmbiguity(
        self, recognizer, dfa: DFA, startIndex: int, stopIndex: int, exact: bool, ambigAlts: set[int], configs: ATNConfigSet
    ): ...
    def reportAttemptingFullContext(
        self, recognizer, dfa: DFA, startIndex: int, stopIndex: int, conflictingAlts: set[int], configs: ATNConfigSet
    ): ...
    def reportContextSensitivity(
        self, recognizer, dfa: DFA, startIndex: int, stopIndex: int, prediction: int, configs: ATNConfigSet
    ): ...
    def getDecisionDescription(self, recognizer, dfa: DFA): ...
    def getConflictingAlts(self, reportedAlts: set[int], configs: ATNConfigSet): ...



from _typeshed import Incomplete
from collections.abc import Callable
from concurrent import futures
from typing import Final, overload
from typing_extensions import Self

from grpc import ServicerContext
from grpc_health.v1 import health_pb2, health_pb2_grpc

SERVICE_NAME: Final[str]
OVERALL_HEALTH: Final[str]

class _Watcher:
    def __init__(self) -> None: ...
    def __iter__(self) -> Self: ...
    def next(self) -> health_pb2.HealthCheckResponse: ...
    def __next__(self) -> health_pb2.HealthCheckResponse: ...
    def add(self, response: health_pb2.HealthCheckResponse) -> None: ...
    def close(self) -> None: ...

class HealthServicer(health_pb2_grpc.HealthServicer):
    def __init__(
        self, experimental_non_blocking: bool = True, experimental_thread_pool: futures.ThreadPoolExecutor | None = None
    ) -> None: ...
    def Check(self, request: health_pb2.HealthCheckRequest, context: ServicerContext) -> health_pb2.HealthCheckResponse: ...
    @overload
    def Watch(
        self, request: health_pb2.HealthCheckRequest, context: ServicerContext, send_response_callback: None = None
    ) -> _Watcher: ...
    @overload
    def Watch(
        self, request: health_pb2.HealthCheckRequest, context: ServicerContext, send_response_callback: Callable[..., Incomplete]
    ) -> None: ...
    def set(self, service: str, status: health_pb2.HealthCheckResponse.ServingStatus) -> None: ...
    def enter_graceful_shutdown(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "is_matching",
    "is_maximal_matching",
    "is_perfect_matching",
    "max_weight_matching",
    "min_weight_matching",
    "maximal_matching",
]

@_dispatchable
def maximal_matching(G: Graph[_Node]) -> set[Incomplete]: ...
def matching_dict_to_set(matching: Mapping[Incomplete, Incomplete]) -> set[Incomplete]: ...
@_dispatchable
def is_matching(G: Graph[_Node], matching: dict[Incomplete, Incomplete] | Iterable[Iterable[Incomplete]]) -> bool: ...
@_dispatchable
def is_maximal_matching(G: Graph[_Node], matching: dict[Incomplete, Incomplete] | Iterable[Iterable[Incomplete]]) -> bool: ...
@_dispatchable
def is_perfect_matching(G: Graph[_Node], matching: dict[Incomplete, Incomplete] | Iterable[Iterable[Incomplete]]) -> bool: ...
@_dispatchable
def min_weight_matching(G: Graph[_Node], weight: str | None = "weight") -> set[Incomplete]: ...
@_dispatchable
def max_weight_matching(
    G: Graph[_Node], maxcardinality: bool | None = False, weight: str | None = "weight"
) -> set[Incomplete]: ...



from _typeshed import MaybeNone
from collections.abc import Callable, Iterator
from typing import Any, Final, Generic, TypeVar, type_check_only
from typing_extensions import disjoint_base

__all__ = [
    "LUA_VERSION",
    "LUA_MAXINTEGER",
    "LUA_MININTEGER",
    "LuaRuntime",
    "LuaError",
    "LuaSyntaxError",
    "LuaMemoryError",
    "as_itemgetter",
    "as_attrgetter",
    "lua_type",
    "unpacks_lua_table",
    "unpacks_lua_table_method",
]

LUA_MAXINTEGER: Final[int]
LUA_MININTEGER: Final[int]
LUA_VERSION: Final[tuple[int, int]]

# cyfunction object
as_attrgetter: Callable[[object], object]
as_itemgetter: Callable[[object], object]

# cyfunction object
lua_type: Callable[[object], str | MaybeNone]

# cyfunction object as decorator
unpacks_lua_table: Callable[[Callable[..., Any]], Callable[..., Any]]
unpacks_lua_table_method: Callable[[Callable[..., Any]], Callable[..., Any]]

# inner classes

@type_check_only
class _LuaIter:
    def __iter__(self) -> Iterator[object]: ...

@type_check_only
class _LuaTable:
    def keys(self) -> _LuaIter: ...
    def values(self) -> _LuaIter: ...
    def items(self) -> _LuaIter: ...

@type_check_only
class _LuaNoGC: ...

@type_check_only
class _LuaObject: ...

# classes

_bint = TypeVar("_bint", bool, int)

@disjoint_base
class FastRLock(Generic[_bint]):
    # @classmethod
    # def __init__(cls, /, *args: Any, **kwargs: Any) -> None: ...
    def acquire(self, blocking: _bint = True) -> _bint: ...
    def release(self) -> None: ...
    def __enter__(self) -> _bint: ...
    def __exit__(self, t: object, v: object, tb: object) -> None: ...

class LuaError(Exception): ...
class LuaSyntaxError(LuaError): ...
class LuaMemoryError(LuaError, MemoryError): ...

@disjoint_base
class LuaRuntime:
    lua_implementation: Final[str]
    lua_version: Final[tuple[int, int]]

    # @classmethod
    # def __cinit__(cls, unpack_return_tuples: bool) -> None: ...
    # def add_pending_unref(self, ref: int) -> None: ...
    # def clean_up_pending_unrefs(self) -> int: ...
    def get_max_memory(self, total: bool = False) -> int | MaybeNone: ...
    def get_memory_used(self, total: bool = False) -> int | MaybeNone: ...
    # def reraise_on_exceptions(self) -> int: ...
    # def store_raised_exception(self, L: object, lua_error_msg: str) -> None: ...  # unannotated
    def eval(self, lua_code: str, *args: Any, name: str | None = None, mode: str | None = None) -> object: ...
    def execute(self, lua_code: str, *args: Any, name: str | None = None, mode: str | None = None) -> object: ...
    def compile(self, lua_code: str, name: str | None = None, mode: str | None = None) -> Callable[..., object]: ...
    def require(self, modulename: str) -> object: ...
    def globals(self) -> _LuaTable: ...
    def table(self, *items: Any, **kwargs: Any) -> _LuaTable: ...
    def table_from(self, *args: Any, recursive: bool = ...) -> _LuaTable: ...
    def nogc(self) -> _LuaNoGC: ...
    def gccollect(self) -> None: ...
    def set_max_memory(self, max_memory: int, total: bool = False) -> None: ...
    def set_overflow_handler(self, overflow_handler: Callable[..., None]) -> None: ...
    # def register_py_object(self, cname: str, pyname: str, obj: object) -> int: ...
    # def init_python_lib(self, register_eval: bool, register_builtins: bool) -> int: ...



from _typeshed import Incomplete

from ._monitor import TMonitor as TMonitor, TqdmSynchronisationWarning as TqdmSynchronisationWarning
from ._tqdm_pandas import tqdm_pandas as tqdm_pandas
from .cli import main as main
from .gui import tqdm as tqdm_gui, trange as tgrange
from .notebook import tqdm_notebook as tqdm_notebook_cls
from .std import (
    TqdmDeprecationWarning as TqdmDeprecationWarning,
    TqdmExperimentalWarning as TqdmExperimentalWarning,
    TqdmKeyError as TqdmKeyError,
    TqdmMonitorWarning as TqdmMonitorWarning,
    TqdmTypeError as TqdmTypeError,
    TqdmWarning as TqdmWarning,
    tqdm as tqdm,
    trange as trange,
)
from .version import __version__ as __version__

__all__ = [
    "tqdm",
    "tqdm_gui",
    "trange",
    "tgrange",
    "tqdm_pandas",
    "tqdm_notebook",
    "tnrange",
    "main",
    "TMonitor",
    "TqdmTypeError",
    "TqdmKeyError",
    "TqdmWarning",
    "TqdmDeprecationWarning",
    "TqdmExperimentalWarning",
    "TqdmMonitorWarning",
    "TqdmSynchronisationWarning",
    "__version__",
]

def tqdm_notebook(*args, **kwargs) -> tqdm_notebook_cls[Incomplete]: ...
def tnrange(*args, **kwargs) -> tqdm_notebook_cls[int]: ...
